package lcsb.mapviewer.services.impl;

import java.awt.geom.Point2D;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.mail.MessagingException;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.common.ObjectUtils;
import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.common.exception.InvalidClassException;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.Comment;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.model.map.species.Species;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.persist.dao.map.CommentDao;
import lcsb.mapviewer.services.interfaces.ICommentService;
import lcsb.mapviewer.services.interfaces.IConfigurationService;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.search.ElementIdentifierDetails;
import lcsb.mapviewer.services.search.comment.CommentDetails;
import lcsb.mapviewer.services.search.comment.FullCommentView;
import lcsb.mapviewer.services.search.comment.FullCommentViewFactory;
import lcsb.mapviewer.services.search.data.ElementIdentifier;
import lcsb.mapviewer.services.search.data.ElementIdentifier.ElementIdentifierType;
import lcsb.mapviewer.services.utils.EmailSender;
import lcsb.mapviewer.services.view.CommentView;
import lcsb.mapviewer.services.view.CommentViewFactory;

/**
 * This class is responsible for all services connected with comments
 * functionality.
 * 
 * @author Piotr Gawron
 * 
 */

@Transactional(value = "txManager")
public class CommentService implements ICommentService {

	/**
	 * Max distance between two points on map for which two comments should be
	 * considered as refereing to the same point.
	 */
	private static final double		COMMENT_POINT_DISTANCE_EPSILON					 = 0.02;

	/**
	 * Default class logger.
	 */
	private static Logger					logger																	 = Logger.getLogger(CommentService.class);

	/**
	 * Number of line with content in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_CONTENT_LINE			 = 0;

	/**
	 * Number of line with coordinates of the comment t in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_COORDINATES_LINE	 = 1;

	/**
	 * Number of line with email in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_EMAIL_LINE				 = 2;

	/**
	 * Number of line with project title in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_PROJECT_TITLE_LINE = 3;

	/**
	 * Number of line with user name in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_USER_NAME_LINE		 = 4;

	/**
	 * Number of line with reason of removing in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_REMOVE_REASON_LINE = 5;

	/**
	 * Number of line with identifier of the commented object in serialized
	 * comment.
	 */
	private static final int			COMMENT_SERIALIZATION_IDENTIFIER_LINE		 = 6;

	/**
	 * Number of line with class type of the commented object in serialized
	 * comment.
	 */
	private static final int			COMMENT_SERIALIZATION_CLASS_TYPE_LINE		 = 7;

	/**
	 * Number of line with user login in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_USER_LOGIN_LINE		 = 8;

	/**
	 * Number of line with pinned value in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_PINNED_LINE				 = 9;

	/**
	 * Number of line with deleted value in serialized comment.
	 */
	private static final int			COMMENT_SERIALIZATION_DELETED_LINE			 = 10;

	/**
	 * Encoding used for serialization/deserialization.
	 */
	private static final String		SERIALIZATION_ENCODING									 = "UTF-8";

	/**
	 * Field separator used in serialization.
	 */
	private String								fieldSeparatorInSerialization						 = "\t";

	/**
	 * Data access object fir comments.
	 */
	@Autowired
	private CommentDao						commentDao;

	/**
	 * Service for operations on Model object.
	 */
	@Autowired
	private IModelService					modelService;

	/**
	 * Service for operations on User object.
	 */
	@Autowired
	private IUserService					userService;

	/**
	 * Service for operations on Configuration object.
	 */
	@Autowired
	private IConfigurationService	configurationService;

	/**
	 * Factory object used for creation fo {@link CommentView} objects.
	 */
	@Autowired
	private CommentViewFactory		commentViewFactory;

	@Override
	public Comment addComment(String name, String email, String content, Model model, Point2D coordinates, Object object, boolean pinned, Model submodel) {
		Comment comment = new Comment();
		comment.setName(name);
		comment.setEmail(email);
		comment.setContent(content);
		comment.setModel(model);
		comment.setCoordinates(coordinates);
		if (object != null) {
			comment.setTableName(object.getClass());
			comment.setTableId(ObjectUtils.getIdOfObject(object));
			if (object instanceof Element) {
				comment.setSubmodel(((Element) object).getModel());
			} else if (object instanceof Reaction) {
				comment.setSubmodel(((Reaction) object).getModel());
			} else {
				throw new InvalidClassException("Cannot comment on object of class: " + object.getClass());
			}
		} else {
			comment.setSubmodel(submodel);
		}
		comment.setUser(null);
		comment.setPinned(pinned);
		commentDao.add(comment);

		try {
			EmailSender emailSender = new EmailSender(configurationService);
			emailSender.sendEmail("New comment appeard in the " + model.getProject().getProjectId(), content, model.getNotifyEmail());
		} catch (MessagingException e) {
			logger.error("Problem with sending email.", e);
		}

		return comment;
	}

	/**
	 * This method returns all comments for model agregated by commented elements.
	 * 
	 * @param model
	 *          object to which we are looking for comments
	 * @param pinned
	 *          this parameter defines what kind of comments we want to see:
	 *          <ul>
	 *          <li>null - all comments</li>
	 *          <li>false - comments that are not visible on the map</li>
	 *          <li>true - comments that are visible on the map</li>
	 *          </ul>
	 * @return list of comments grouped by commented elements
	 */
	protected List<List<Comment>> getAgregatedComments(Model model, Boolean pinned) {
		List<Comment> comments = commentDao.getCommentByModel((Model) model, pinned, false);
		List<List<Comment>> result = new ArrayList<List<Comment>>();

		Map<String, List<Comment>> objects = new HashMap<String, List<Comment>>();
		for (Comment comment : comments) {
			if (comment.getTableName() == null) {
				List<Comment> list = new ArrayList<Comment>();
				list.add(comment);
				result.add(list);
			} else {
				String id = comment.getTableId() + "," + comment.getTableName();

				List<Comment> list = objects.get(id);
				if (list == null) {
					list = new ArrayList<Comment>();
					objects.put(id, list);
					result.add(list);
				}
				list.add(comment);
			}
		}
		return result;
	}

	@Override
	public void deleteComment(User loggedUser, String commentId, String reason) {
		int id = -1;
		try {
			id = Integer.parseInt(commentId);

		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		Comment comment = commentDao.getById(id);
		if (comment == null) {
			logger.error("Invalid comment id: " + commentId);
		} else {
			comment.setDeleted(true);
			comment.setRemoveReason(reason);
			commentDao.update(comment);
		}
	}

	@Override
	public List<CommentView> getCommentsByMap(Model model) {
		List<Comment> comments = commentDao.getCommentByModel(model, null, null);
		List<CommentView> commentList = new ArrayList<CommentView>();
		for (Comment comment : comments) {
			CommentView row = commentViewFactory.create(comment, model);
			commentList.add(row);
		}
		return commentList;
	}

	@Override
	public long getCommentCount() {
		return commentDao.getCount();
	}

	@Override
	public String createCustomSerializationOfComment(Comment comment) {
		Model model = comment.getModelData().getModel();
		String result = "";
		try {
			result += URLEncoder.encode(comment.getContent(), SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			result += URLEncoder.encode(comment.getCoordinates().getX() + "," + comment.getCoordinates().getY(), SERIALIZATION_ENCODING)
					+ fieldSeparatorInSerialization;
			result += URLEncoder.encode(comment.getEmail(), SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			result += URLEncoder.encode(comment.getModelData().getProject().getProjectId() + "," + comment.getModelData().getMapVersion(), SERIALIZATION_ENCODING)
					+ fieldSeparatorInSerialization;
			result += URLEncoder.encode(comment.getName(), SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			result += URLEncoder.encode(comment.getRemoveReason(), SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;

			String identifier = "";
			if (comment.getTableName() != null) {
				if (comment.getTableName().getName().contains("Reaction")) {
					Reaction reaction = model.getReactionByDbId(comment.getTableId());
					if (reaction != null) {
						identifier = reaction.getIdReaction();
					} else {
						logger.warn("Invalid reaction dbID: " + comment.getTableId() + ". Table: " + comment.getTableName().getName());
					}
				} else {
					Element alias = model.getElementByDbId(comment.getTableId());
					if (alias != null) {
						identifier = ((Species) alias).getElementId();
					} else {
						logger.warn("Invalid alias dbID: " + comment.getTableId());
					}
				}
			}
			result += URLEncoder.encode(identifier, SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			if (comment.getTableName() == null) {
				result += URLEncoder.encode("", SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			} else {
				result += URLEncoder.encode(comment.getTableName().getCanonicalName(), SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			}
			if (comment.getUser() == null) {
				result += URLEncoder.encode("", SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			} else {
				result += URLEncoder.encode(comment.getUser().getLogin(), SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			}
			if (comment.isPinned()) {
				result += URLEncoder.encode("true", SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			} else {
				result += URLEncoder.encode("false", SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			}

			if (comment.isDeleted()) {
				result += URLEncoder.encode("true", SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			} else {
				result += URLEncoder.encode("false", SERIALIZATION_ENCODING) + fieldSeparatorInSerialization;
			}

			result += "eof" + fieldSeparatorInSerialization;
		} catch (UnsupportedEncodingException e) {
			logger.error("UnsupportedEncodingException: " + SERIALIZATION_ENCODING);
		}
		return result;
	}

	@Override
	public Comment deserializeCustomSerializationOfComment(String serializedComment) {
		Comment result = new Comment();
		String[] lines = serializedComment.split(fieldSeparatorInSerialization);
		try {
			result.setContent(URLDecoder.decode(lines[COMMENT_SERIALIZATION_CONTENT_LINE], SERIALIZATION_ENCODING));
			String[] params = URLDecoder.decode(lines[COMMENT_SERIALIZATION_COORDINATES_LINE], SERIALIZATION_ENCODING).split(",");
			String xStr = params[0];
			String yStr = params[1];
			double x = Double.parseDouble(xStr);
			double y = Double.parseDouble(yStr);
			result.setCoordinates(new Point2D.Double(x, y));
			result.setEmail(URLDecoder.decode(lines[COMMENT_SERIALIZATION_EMAIL_LINE], SERIALIZATION_ENCODING));
			String projectName = URLDecoder.decode(lines[COMMENT_SERIALIZATION_PROJECT_TITLE_LINE], SERIALIZATION_ENCODING).split(",")[0];
			String mapVersion = URLDecoder.decode(lines[COMMENT_SERIALIZATION_PROJECT_TITLE_LINE], SERIALIZATION_ENCODING).split(",")[1];
			Model model = (Model) modelService.getLastModelByProjectId(projectName);
			if (model != null) {
				if (model.getMapVersion().equals(mapVersion)) {
					result.setModel(model);
				} else {
					throw new NotImplementedException();
				}
			} else {
				throw new InvalidArgumentException("Unknown project: " + projectName);
			}
			result.setName(URLDecoder.decode(lines[COMMENT_SERIALIZATION_USER_NAME_LINE], SERIALIZATION_ENCODING));
			result.setRemoveReason(URLDecoder.decode(lines[COMMENT_SERIALIZATION_REMOVE_REASON_LINE], SERIALIZATION_ENCODING));
			String id = URLDecoder.decode(lines[COMMENT_SERIALIZATION_IDENTIFIER_LINE], SERIALIZATION_ENCODING);
			if (id.startsWith("re")) {
				Reaction reaction = model.getReactionByReactionId(id);
				if (reaction == null) {
					logger.error("Unknown reaction: " + id);
				} else {
					Integer dbId = model.getReactionByReactionId(id).getId();
					result.setTableId(dbId);
				}
			} else if (id.startsWith("sa")) {
				Integer dbId = model.getElementByElementId(id).getId();
				result.setTableId(dbId);
			}
			String className = URLDecoder.decode(lines[COMMENT_SERIALIZATION_CLASS_TYPE_LINE], SERIALIZATION_ENCODING);
			if (className != null && !className.equals("")) {
				try {
					Class<?> clazz = Class.forName(className);
					result.setTableName(clazz);
				} catch (ClassNotFoundException e) {
					logger.error("Class not found: " + className);
				}
			}
			String login = URLDecoder.decode(lines[COMMENT_SERIALIZATION_USER_LOGIN_LINE], SERIALIZATION_ENCODING);
			if (login == null || login.equals("null")) {
				User user = userService.getUserByLogin(login);
				result.setUser(user);
			}
			result.setPinned(URLDecoder.decode(lines[COMMENT_SERIALIZATION_PINNED_LINE], SERIALIZATION_ENCODING).equalsIgnoreCase("true"));
			result.setDeleted(URLDecoder.decode(lines[COMMENT_SERIALIZATION_DELETED_LINE], SERIALIZATION_ENCODING).equalsIgnoreCase("true"));

		} catch (UnsupportedEncodingException e) {
			logger.error("UnsupportedEncodingException: " + SERIALIZATION_ENCODING);
		}
		return result;
	}

	@Override
	public void removeCommentsForModel(Model model) {
		List<Comment> comments = commentDao.getCommentByModel(model, null, null);
		for (Comment comment : comments) {
			commentDao.delete(comment);
		}
	}

	/**
	 * @return the commentDao
	 * @see #commentDao
	 */
	public CommentDao getCommentDao() {
		return commentDao;
	}

	/**
	 * @param commentDao
	 *          the commentDao to set
	 * @see #commentDao
	 */
	public void setCommentDao(CommentDao commentDao) {
		this.commentDao = commentDao;
	}

	/**
	 * @return the modelService
	 * @see #modelService
	 */
	public IModelService getModelService() {
		return modelService;
	}

	/**
	 * @param modelService
	 *          the modelService to set
	 * @see #modelService
	 */
	public void setModelService(IModelService modelService) {
		this.modelService = modelService;
	}

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	/**
	 * @return the configurationService
	 * @see #configurationService
	 */
	public IConfigurationService getConfigurationService() {
		return configurationService;
	}

	/**
	 * @param configurationService
	 *          the configurationService to set
	 * @see #configurationService
	 */
	public void setConfigurationService(IConfigurationService configurationService) {
		this.configurationService = configurationService;
	}

	/**
	 * Creates list of {@link CustomCommentMarker} that represents agregated
	 * comments.
	 * 
	 * @param comments
	 *          agregated comments (every list contains comments for single
	 *          element)
	 * @param topModel
	 *          model which is commented
	 * @return list of {@link CustomCommentMarker} that represents agregated
	 *         comments
	 */
	protected List<FullCommentView> agregatedCommentsIntoMarker(List<List<Comment>> comments, Model topModel) {
		List<FullCommentView> result = new ArrayList<>();
		FullCommentViewFactory factory = new FullCommentViewFactory();
		for (List<Comment> list : comments) {
			result.add(factory.create(list));
		}
		return result;
	}

	@Override
	public List<FullCommentView> getCommentMarkers(Model model) {
		List<List<Comment>> comments = getAgregatedComments(model, true);
		return agregatedCommentsIntoMarker(comments, model);
	}

	@Override
	public void removeCommentsForModel(ModelData model) {
		List<Comment> comments = commentDao.getCommentByModel(model, null, null);
		for (Comment comment : comments) {
			commentDao.delete(comment);
		}
	}

	/**
	 * Returns {@link CommentDetails detailed information} about comments for a
	 * specified element.
	 * 
	 * @param element
	 *          element for which we are looking for detailed information
	 * @param model
	 *          model where we are looking for the information
	 * @return {@link CommentDetails detailed information} about comments for a
	 *         specified element
	 */
	public List<ElementIdentifierDetails> getElementInformationForResult(ElementIdentifier element, Model model) {
		// TODO refactor!!!!
		// it's slow...
		List<FullCommentView> comments = getCommentMarkers(model);

		List<ElementIdentifierDetails> result = new ArrayList<>();

		if (ElementIdentifierType.REACTION.getJsName().equalsIgnoreCase(element.getType())) {
			for (FullCommentView comment : comments) {
				if (Reaction.class.equals(comment.getType())) {
					if (comment.getIdObject().equals(element.getObjectId()) && comment.getModelId().equals(element.getModelId())) {
						for (Pair<String, String> c : comment.getComments()) {
							result.add(new CommentDetails(c.getLeft(), c.getRight()));
						}
						return result;
					}
				}
			}
			return result;
		} else if (ElementIdentifierType.POINT.getJsName().equalsIgnoreCase(element.getType())) {
			for (FullCommentView comment : comments) {
				if (Point2D.class.equals(comment.getType())) {
					if (equalPoints(comment.getUniqueId(), element.getObjectId())) {
						for (Pair<String, String> c : comment.getComments()) {
							result.add(new CommentDetails(c.getLeft(), c.getRight()));
						}
						return result;
					}
				}
			}
			return result;
		} else if (ElementIdentifierType.ALIAS.getJsName().equalsIgnoreCase(element.getType())) {
			for (FullCommentView comment : comments) {
				if (Element.class.equals(comment.getType())) {
					if (comment.getIdObject().equals(element.getObjectId()) && comment.getModelId().equals(element.getModelId())) {
						for (Pair<String, String> c : comment.getComments()) {
							result.add(new CommentDetails(c.getLeft(), c.getRight()));
						}
						return result;
					}
				}
			}
			return result;
		} else {
			throw new InvalidArgumentException("Unknown clas stype: " + element.getType());
		}
	}

	/**
	 * Checks if identifier of the points refer to the same point.
	 * 
	 * @param uniqueId
	 *          identifier assigned by the java code
	 * @param objectId
	 *          identifier assigned by the javascript code (browser)
	 * @return <code>true</code> if both identifiers refer to the same point,
	 *         <code>false</code> otherwise
	 */
	protected boolean equalPoints(String uniqueId, String objectId) {
		String tmp = uniqueId.substring(uniqueId.indexOf("[") + 1, uniqueId.indexOf("]"));
		String[] ids = tmp.split(",");
		Double x1 = Double.valueOf(ids[0].trim());
		Double y1 = Double.valueOf(ids[1].trim());

		tmp = objectId.substring(objectId.indexOf("(") + 1, objectId.indexOf(")"));
		ids = tmp.split(",");
		Double x2 = Double.valueOf(ids[0].trim());
		Double y2 = Double.valueOf(ids[1].trim());

		if (Math.abs(x1 - x2) > COMMENT_POINT_DISTANCE_EPSILON || Math.abs(y1 - y2) > COMMENT_POINT_DISTANCE_EPSILON) {
			return false;
		}
		return true;
	}

}
