package lcsb.mapviewer.services.interfaces;

import java.util.List;

import lcsb.mapviewer.common.IProgressUpdater;
import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.services.search.data.FullAliasView;
import lcsb.mapviewer.services.search.data.LightAliasView;
import lcsb.mapviewer.services.search.data.LightReactionView;
import lcsb.mapviewer.services.view.ModelView;
import lcsb.mapviewer.services.view.ProjectView;

/**
 * Service that manages models (maps).
 * 
 * @author Piotr Gawron
 * 
 */
public interface IModelService {

	/**
	 * Returns the newest model for the project identified by the
	 * {@link lcsb.mapviewer.model.Project#projectId project identifier}.
	 * 
	 * @param projectId
	 *          {@link lcsb.mapviewer.model.Project#projectId project identifier}
	 * @return the newest model for the project
	 */
	Model getLastModelByProjectId(String projectId);

	/**
	 * Caches information about all pubmed articles for the given model.
	 * 
	 * @param model
	 *          model in which we are caching the data
	 * @param updater
	 *          callback interface that update progress information
	 */
	void cacheAllPubmedIds(Model model, IProgressUpdater updater);

	/**
	 * Updates information in the model with the one taken from selectedProject.
	 * 
	 * @param model
	 *          model to update
	 * @param selectedProject
	 *          source of data that should be used in the update
	 */
	void updateModel(ModelData model, ProjectView selectedProject);

	/**
	 * Removes model from application cache.
	 * 
	 * @param model
	 *          model to remove
	 */
	void removeModelFromCache(Model model);

	/**
	 * Caches information about all miriam resources in the model.
	 * 
	 * @param model
	 *          model in which we are caching the data
	 * @param updater
	 *          callback interface that update progress information
	 */
	void cacheAllMiriamLinks(Model model, IProgressUpdater updater);

	/**
	 * Returns {@link ModelView} for the given model.
	 * 
	 * @param model
	 *          model fgor which view is returned.
	 * @param user
	 *          user for which custom layouts should be added
	 * @return {@link ModelView} for the given {@link Model}
	 */
	ModelView getModelView(Model model, User user);

	/**
	 * Removes model from application cache.
	 * 
	 * @param model
	 *          model to remove
	 */
	void removeModelFromCache(ModelData model);

	/**
	 * Removes model from application cache.
	 * 
	 * @param projectId
	 *          identifier of the project
	 */
	void removeModelFromCacheByProjectId(String projectId);

	/**
	 * Returns list of {@link LightAliasView aliases} for given identifiers.
	 * 
	 * @param model
	 *          model where aliases are located
	 * @param identifiers
	 *          list of alias identifiers in a given submodel. Every {@link Pair}
	 *          contains information about {@link Model#getId() model identifier}
	 *          (in {@link Pair#left}) and
	 *          {@link lcsb.mapviewer.model.map.species.Element#getId() alias
	 *          identifier} (in {@link Pair#right}).
	 * @return list of {@link LightAliasView aliases} for given identifiers
	 */
	List<LightAliasView> getLightAliasesByIds(Model model, List<Pair<Integer, Integer>> identifiers);

	/**
	 * Returns list of {@link LightReactionView reactions} for given identifiers.
	 * 
	 * @param model
	 *          model where aliases are located
	 * @param identifiers
	 *          list of reaction identifiers in a given submodel. Every
	 *          {@link Pair} contains information about {@link Model#getId() model
	 *          identifier} (in {@link Pair#left}) and
	 *          {@link lcsb.mapviewer.model.map.reaction.Reaction#getId() reaction
	 *          identifier} (in {@link Pair#right}).
	 * @return list of {@link LightReactionView reactions} for given identifiers
	 */
	List<LightReactionView> getLightReactionsByIds(Model model, List<Pair<Integer, Integer>> identifiers);

	/**
	 * Returns list of {@link FullAliasView aliases} (with full information) for
	 * given identifiers.
	 * 
	 * @param model
	 *          model where aliases are located
	 * @param identifiers
	 *          list of alias identifiers in a given submodel. Every {@link Pair}
	 *          contains information about {@link Model#getId() model identifier}
	 *          (in {@link Pair#left}) and
	 *          {@link lcsb.mapviewer.model.map.species.Element#getId() alias
	 *          identifier} (in {@link Pair#right}).
	 * @return list of {@link FullAliasView aliases} (with full information about
	 *         aliases) for given identifiers
	 */
	List<FullAliasView> getFullAliasesByIds(Model model, List<Pair<Integer, Integer>> identifiers);
}
