package lcsb.mapviewer.services.interfaces;

import java.util.Collection;
import java.util.List;

import lcsb.mapviewer.commands.ColorExtractor;
import lcsb.mapviewer.model.user.BasicPrivilege;
import lcsb.mapviewer.model.user.PrivilegeType;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.services.view.UserView;

/**
 * Service that manages users.
 * 
 * @author Piotr Gawron
 * 
 */
public interface IUserService {
	/**
	 * Check if its possible to login using given login and password.
	 * 
	 * @param login
	 *          user login
	 * @param password
	 *          plan password
	 * @return <code>true</code> if credentials are ok, <code>false</code>
	 *         otherwise
	 */
	boolean login(String login, String password);

	/**
	 * Returns user by login.
	 * 
	 * @param login
	 *          user login
	 * @return user if the login is valid, <code>null</code> otherwise
	 */
	User getUserByLogin(String login);

	/**
	 * Checks if user has a given privilege.
	 * 
	 * @param user
	 *          user for which the check is performed
	 * @param type
	 *          type of the privilege
	 * @return <code>true</code> if user has privilege, <code>false</code>
	 *         otherwise
	 */
	boolean userHasPrivilege(User user, PrivilegeType type);

	/**
	 * Returns level of the user privilege.
	 * 
	 * @param user
	 *          user for which the check is performed
	 * @param type
	 *          type of the privilege
	 * @return level of the privilege
	 */
	int getUserPrivilegeLevel(User user, PrivilegeType type);

	/**
	 * Checks if user has a given privilege on the object.
	 * 
	 * @param user
	 *          user for which the check is performed
	 * @param type
	 *          type of the privilege
	 * @param object
	 *          object of the privilege
	 * @return <code>true</code> if user has privilege, <code>false</code>
	 *         otherwise
	 */
	boolean userHasPrivilege(User user, PrivilegeType type, Object object);

	/**
	 * Returns level of the user privilege.
	 * 
	 * @param object
	 *          object of the privilege
	 * @param user
	 *          user for which the check is performed
	 * @param type
	 *          type of the privilege
	 * @return level of the privilege
	 */
	int getUserPrivilegeLevel(User user, PrivilegeType type, Object object);

	/**
	 * Sets the user privilege.
	 * 
	 * @param user
	 *          user for whom the privilege should be granted/dropped
	 * @param privilege
	 *          user privilege
	 */
	void setUserPrivilege(User user, BasicPrivilege privilege);

	/**
	 * Adds user to the system.
	 * 
	 * @param user
	 *          user to be added
	 */
	void addUser(User user);

	/**
	 * Updates user in the system.
	 * 
	 * @param user
	 *          user to be updated
	 */
	void updateUser(User user);

	/**
	 * Removes user from the system.
	 * 
	 * @param user
	 *          user to be removed
	 */
	void deleteUser(User user);

	/**
	 * Returns user by the database identifier.
	 * 
	 * @param id
	 *          database identifier
	 * @return user for the given identifier, null if user doesn't exist
	 * 
	 */
	User getUserById(int id);

	/**
	 * Returns user views for all users.
	 * 
	 * @return list of user {@link lcsb.mapviewer.services.view.AbstractView
	 *         views} for all users
	 */
	List<UserView> getAllUserRows();

	/**
	 * @param user
	 *          user that will have view.
	 * @return view for one user.
	 */
	UserView getUserRow(User user);

	/**
	 * Updates user from user {@link lcsb.mapviewer.services.view.AbstractView
	 * view}.
	 * 
	 * @param selectedUser
	 *          user {@link lcsb.mapviewer.services.view.AbstractView view} to
	 *          update
	 */
	void updateUser(UserView selectedUser);

	/**
	 * Removes user based on user {@link lcsb.mapviewer.services.view.AbstractView
	 * view}.
	 * 
	 * @param selectedUser
	 *          user {@link lcsb.mapviewer.services.view.AbstractView view} to be
	 *          removed
	 */
	void deleteUser(UserView selectedUser);

	/**
	 * Creates empty view.
	 * 
	 * @return empty user {@link lcsb.mapviewer.services.view.AbstractView view} .
	 */
	UserView createEmptyUserRow();

	/**
	 * Drops privileges for every user on the given object and type.
	 * 
	 * @param type
	 *          type of privilege that should be dropped
	 * @param objectId
	 *          identifier of the object to which privileges should be dropped
	 */
	void dropPrivilegesForObjectType(PrivilegeType type, int objectId);

	/**
	 * Adds user from user view.
	 * 
	 * @param selectedUser
	 *          user view representation that should be added
	 */
	void addUser(UserView selectedUser);

	/**
	 * Adds privilege to the user.
	 * 
	 * @param user
	 *          user to which privilege should be added
	 * @param type
	 *          type of the privilege
	 */
	void setUserPrivilege(User user, PrivilegeType type);

	/**
	 * Updates users from list of user
	 * {@link lcsb.mapviewer.services.view.AbstractView view}.
	 * 
	 * @param users
	 *          user {@link lcsb.mapviewer.services.view.AbstractView views} to
	 *          update
	 */
	void updateUsers(Collection<UserView> users);

	/**
	 * @param password
	 *          input password
	 * @return encoded password
	 */
	String encodePassword(String password);

	/**
	 * Returns {@link User} for given "name surname" string.
	 * 
	 * @param nameSurnameString
	 *          string identifing user with name and surname separated by single
	 *          space
	 * @return {@link User} for given "name surname" string
	 */
	User getUserByNameSurname(String nameSurnameString);

	/**
	 * Returns {@link ColorExtractor} that transform overlay values into colors
	 * for given user.
	 * 
	 * @param user
	 *          {@link User} for which {@link ColorExtractor} will be obtained
	 * @return {@link ColorExtractor} that transform overlay values into colors
	 *         for given user
	 */
	ColorExtractor getColorExtractorForUser(User user);
}
