package lcsb.mapviewer.services.search.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.services.search.IHeavyView;
import lcsb.mapviewer.services.view.AnnotationView;

/**
 * This class represent data that are sent into the client (by design Google Map
 * object, but not necessery). It is connected to the single reaction on the
 * map. It contains all information required for vizualization:
 * <ul>
 * <li>localization information,</li>
 * <li>associated icon</li>
 * <li>some structural information.</li>
 * </ul>
 * However, there are no information about formating, css, etc.
 * 
 * @author Piotr Gawron
 * 
 */
public class FullReactionView extends LightReactionView implements IHeavyView {

	/**
	 * 
	 */
	private static final long		 serialVersionUID					 = 1L;

	/**
	 * List of reaction annotations.
	 */
	private List<AnnotationView> references								 = new ArrayList<AnnotationView>();

	/**
	 * List of markers to which this reaction refers to.
	 */
	private List<FullAliasView>	 elements									 = new ArrayList<FullAliasView>();

	/**
	 * Submodel to which this reaction points.
	 */
	private String							 modelName								 = null;

	/**
	 * Symbol of the reaction (RECON).
	 * 
	 * @see Reaction#symbol
	 */
	private String							 symbol										 = null;

	/**
	 * Abbreviation of the reaction (RECON).
	 * 
	 * @see Reaction#abbreviation
	 */
	private String							 abbreviation							 = null;

	/**
	 * Formula (RECON).
	 * 
	 * @see Reaction#formula
	 */
	private String							 formula									 = null;

	/**
	 * Mechanical confidence score (RECON).
	 * 
	 * @see Reaction#mechanicalConfidenceScore
	 */
	private Integer							 mechanicalConfidenceScore = null;

	/**
	 * Lower bound (RECON).
	 * 
	 * @see Reaction#lowerBound
	 */
	private Double							 lowerBound								 = null;

	/**
	 * Upper bound (RECON).
	 * 
	 * @see Reaction#upperBound
	 */
	private Double							 upperBound								 = null;

	/**
	 * Subsystem (RECON).
	 * 
	 * @see Reaction#subsystem
	 */
	private String							 subsystem								 = null;

	/**
	 * Notes about reaction.
	 * 
	 * @see Reaction#notes
	 */
	private String							 notes										 = null;

	/**
	 * Name of reaction.
	 */
	private String							 name											 = null;

	/**
	 * Gene protein reaction (RECON).
	 * 
	 * @see Reaction#geneProteinReaction
	 */
	private String							 geneProteinReaction			 = null;

	/**
	 * Lists of all synonyms used for describing this element.
	 * 
	 * @see Reaction#synonyms
	 */
	private List<String>				 synonyms									 = new ArrayList<String>();

	/**
	 * List of other unstructurized feuters, specific for some subclases of Alias.
	 * 
	 */
	private Map<String, Object>	 other										 = new HashMap<>();

	/**
	 * Default constructor with localization passed in the argument.
	 * 
	 * @param reaction
	 *          original reaction from which this marker is creted
	 */
	protected FullReactionView(Reaction reaction) {
		super(reaction);
		setModelName(reaction.getModel().getName());
		setNotes("Type: " + reaction.getStringType() + "\n" + reaction.getNotes().trim());
		setName("Reaction: " + reaction.getIdReaction());
		if (reaction.getModelData() != null) {
			setModelId(reaction.getModelData().getId());
		}
		symbol = reaction.getSymbol();
		abbreviation = reaction.getAbbreviation();
		formula = reaction.getFormula();
		mechanicalConfidenceScore = reaction.getMechanicalConfidenceScore();
		lowerBound = reaction.getLowerBound();
		upperBound = reaction.getUpperBound();
		subsystem = reaction.getSubsystem();
		geneProteinReaction = reaction.getGeneProteinReaction();
		synonyms.addAll(reaction.getSynonyms());
	}

	/**
	 * Constructor that should be used only for (de)serialization of the object.
	 */
	protected FullReactionView() {

	}

	@Override
	public String toString() {
		return this.getClass() + " [name = " + getName() + "]";
	}

	/**
	 * @return the symbol
	 * @see #symbol
	 */
	public String getSymbol() {
		return symbol;
	}

	/**
	 * @param symbol
	 *          the symbol to set
	 * @see #symbol
	 */
	public void setSymbol(String symbol) {
		this.symbol = symbol;
	}

	/**
	 * @return the abbreviation
	 * @see #abbreviation
	 */
	public String getAbbreviation() {
		return abbreviation;
	}

	/**
	 * @param abbreviation
	 *          the abbreviation to set
	 * @see #abbreviation
	 */
	public void setAbbreviation(String abbreviation) {
		this.abbreviation = abbreviation;
	}

	/**
	 * @return the formula
	 * @see #formula
	 */
	public String getFormula() {
		return formula;
	}

	/**
	 * @param formula
	 *          the formula to set
	 * @see #formula
	 */
	public void setFormula(String formula) {
		this.formula = formula;
	}

	/**
	 * @return the mechanicalConfidenceScore
	 * @see #mechanicalConfidenceScore
	 */
	public Integer getMechanicalConfidenceScore() {
		return mechanicalConfidenceScore;
	}

	/**
	 * @param mechanicalConfidenceScore
	 *          the mechanicalConfidenceScore to set
	 * @see #mechanicalConfidenceScore
	 */
	public void setMechanicalConfidenceScore(Integer mechanicalConfidenceScore) {
		this.mechanicalConfidenceScore = mechanicalConfidenceScore;
	}

	/**
	 * @return the lowerBound
	 * @see #lowerBound
	 */
	public Double getLowerBound() {
		return lowerBound;
	}

	/**
	 * @param lowerBound
	 *          the lowerBound to set
	 * @see #lowerBound
	 */
	public void setLowerBound(Double lowerBound) {
		this.lowerBound = lowerBound;
	}

	/**
	 * @return the upperBound
	 * @see #upperBound
	 */
	public Double getUpperBound() {
		return upperBound;
	}

	/**
	 * @param upperBound
	 *          the upperBound to set
	 * @see #upperBound
	 */
	public void setUpperBound(Double upperBound) {
		this.upperBound = upperBound;
	}

	/**
	 * @return the subsystem
	 * @see #subsystem
	 */
	public String getSubsystem() {
		return subsystem;
	}

	/**
	 * @param subsystem
	 *          the subsystem to set
	 * @see #subsystem
	 */
	public void setSubsystem(String subsystem) {
		this.subsystem = subsystem;
	}

	/**
	 * @return the geneProteinReaction
	 * @see #geneProteinReaction
	 */
	public String getGeneProteinReaction() {
		return geneProteinReaction;
	}

	/**
	 * @param geneProteinReaction
	 *          the geneProteinReaction to set
	 * @see #geneProteinReaction
	 */
	public void setGeneProteinReaction(String geneProteinReaction) {
		this.geneProteinReaction = geneProteinReaction;
	}

	/**
	 * @return the synonyms
	 * @see #synonyms
	 */
	public List<String> getSynonyms() {
		return synonyms;
	}

	/**
	 * @param synonyms
	 *          the synonyms to set
	 * @see #synonyms
	 */
	public void setSynonyms(List<String> synonyms) {
		this.synonyms = synonyms;
	}

	/**
	 * @return the description
	 * @see #description
	 */
	public String getNotes() {
		return notes;
	}

	/**
	 * @param description
	 *          the description to set
	 * @see #description
	 */
	public void setNotes(String description) {
		this.notes = description;
	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          the name to set
	 * @see #name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the modelName
	 * @see #modelName
	 */
	public String getModelName() {
		return modelName;
	}

	/**
	 * @param modelName
	 *          the modelName to set
	 * @see #modelName
	 */
	public void setModelName(String modelName) {
		this.modelName = modelName;
	}

	/**
	 * @return the elements
	 * @see #elements
	 */
	public List<FullAliasView> getElements() {
		return elements;
	}

	/**
	 * @param elements
	 *          the elements to set
	 * @see #elements
	 */
	public void setElements(List<FullAliasView> elements) {
		this.elements = elements;
	}

	/**
	 * @return the references
	 * @see #references
	 */
	public List<AnnotationView> getReferences() {
		return references;
	}

	/**
	 * @param references
	 *          the references to set
	 * @see #references
	 */
	public void setReferences(List<AnnotationView> references) {
		this.references = references;
	}

	/**
	 * @return the other
	 * @see #other
	 */
	public Map<String, Object> getOther() {
		return other;
	}

	/**
	 * @param other
	 *          the other to set
	 * @see #other
	 */
	public void setOther(Map<String, Object> other) {
		this.other = other;
	}

}
