package lcsb.mapviewer.services.search.data;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import lcsb.mapviewer.model.map.reaction.Modifier;
import lcsb.mapviewer.model.map.reaction.NodeOperator;
import lcsb.mapviewer.model.map.reaction.Product;
import lcsb.mapviewer.model.map.reaction.Reactant;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.services.search.ElementView;
import lcsb.mapviewer.services.search.ILightView;

/**
 * Light {@link ElementView view} for alias representing data that should be
 * send to client.
 * 
 * @author Piotr Gawron
 *
 */
public class LightReactionView extends ElementView implements ILightView {

	/**
	 * 
	 */
	private static final long	serialVersionUID = 1L;

	/**
	 * List of lines that depict the reaction.
	 */
	private List<LightLine>		lines						 = new ArrayList<>();

	/**
	 * Central point of the reaction.
	 */
	private Point2D						centerPoint;

	/**
	 * Default constructor.
	 * 
	 * @param reaction
	 *          original {@link Reaction} from which the object is created
	 */
	public LightReactionView(Reaction reaction) {
		super(reaction.getId(), reaction.getModel().getId());
		if (reaction != null) {
			for (Reactant reactant : reaction.getReactants()) {
				addReactantLines(reactant);
			}
			for (Product product : reaction.getProducts()) {
				addProductLines(product);
			}
			for (Modifier modifier : reaction.getModifiers()) {
				addModifierLines(modifier);
			}
			for (NodeOperator operator : reaction.getOperators()) {
				addOperatorLines(operator);
			}
			setCenterPoint(reaction.getCenterPoint());
		}
	}

	/**
	 * Transform {@link NodeOperator} into set of {@link LightLine lines} and put
	 * them into {@link #lines} field.
	 * 
	 * @param operator
	 *          operator to be transformed
	 */
	private void addOperatorLines(NodeOperator operator) {
		List<Line2D> rLines = operator.getLine().getLines();
		for (int i = 0; i < rLines.size(); i++) {
			LightLine l = new LightLine(rLines.get(i));
			l.setType(LightLineType.MIDDLE);
			lines.add(l);
		}
	}

	/**
	 * Transform {@link Modifier} into set of {@link LightLine lines} and put them
	 * into {@link #lines} field.
	 * 
	 * @param modifier
	 *          modifier to be transformed
	 */
	private void addModifierLines(Modifier modifier) {
		List<Line2D> rLines = modifier.getLine().getLines();
		if (rLines.size() > 0) {
			LightLine l = new LightLine(rLines.get(rLines.size() - 1));
			l.setType(LightLineType.MIDDLE);
			lines.add(l);
			for (int i = 0; i < rLines.size() - 1; i++) {
				l = new LightLine(rLines.get(i));
				l.setType(LightLineType.MIDDLE);
				lines.add(l);
			}
		}
	}

	/**
	 * Transform {@link Product} into set of {@link LightLine lines} and put them
	 * into {@link #lines} field.
	 * 
	 * @param product
	 *          product to be transformed
	 */
	private void addProductLines(Product product) {
		List<Line2D> rLines = product.getLine().getLines();
		if (rLines.size() > 0) {
			LightLine l = new LightLine(rLines.get(rLines.size() - 1));
			l.setType(LightLineType.END);
			lines.add(l);
			for (int i = 0; i < rLines.size() - 1; i++) {
				l = new LightLine(rLines.get(i));
				l.setType(LightLineType.MIDDLE);
				lines.add(l);
			}
		}
	}

	/**
	 * Transform {@link Reactant} into set of {@link LightLine lines} and put them
	 * into {@link #lines} field.
	 * 
	 * @param reactant
	 *          reactant to be transformed
	 */
	public void addReactantLines(Reactant reactant) {
		List<Line2D> rLines = reactant.getLine().getLines();
		if (rLines.size() > 0) {
			LightLine l = new LightLine(rLines.get(0));
			l.setType(LightLineType.START);
			lines.add(l);
			for (int i = 1; i < rLines.size(); i++) {
				l = new LightLine(rLines.get(i));
				l.setType(LightLineType.MIDDLE);
				lines.add(l);
			}
		}
	}

	/**
	 * Constructor that should be used only for (de)serialization.
	 */
	protected LightReactionView() {
		super();
	}

	/**
	 * Contructor that creates object from {@link FullReactionView} by copying the
	 * values from it.
	 * 
	 * @param original
	 *          original object
	 */
	protected LightReactionView(FullReactionView original) {
		super(original.getUniqueId(), original.getModelId());
		for (LightLine line : original.getLines()) {
			lines.add(new LightLine(line));
		}
	}

	/**
	 * @return the lines
	 * @see #lines
	 */
	public List<LightLine> getLines() {
		return lines;
	}

	/**
	 * @param lines
	 *          the lines to set
	 * @see #lines
	 */
	public void setLines(List<LightLine> lines) {
		this.lines = lines;
	}

	/**
	 * @return the centerPoint
	 * @see #centerPoint
	 */
	public Point2D getCenterPoint() {
		return centerPoint;
	}

	/**
	 * @param centerPoint
	 *          the centerPoint to set
	 * @see #centerPoint
	 */
	public void setCenterPoint(Point2D centerPoint) {
		this.centerPoint = centerPoint;
	}
}
