package lcsb.mapviewer.services.search.db;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.annotation.data.Article;
import lcsb.mapviewer.annotation.data.ArticleCitationComparator;
import lcsb.mapviewer.annotation.data.Target;
import lcsb.mapviewer.annotation.data.TargetType;
import lcsb.mapviewer.annotation.services.PubmedParser;
import lcsb.mapviewer.annotation.services.PubmedSearchException;
import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.AnnotatedObject;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelSubmodelConnection;
import lcsb.mapviewer.services.view.AbstractViewFactory;
import lcsb.mapviewer.services.view.AnnotationViewFactory;

/**
 * Factory class for {@link TargetView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class TargetViewFactory extends AbstractViewFactory<Target, TargetView> {

	/**
	 * Default class logger.
	 */
	private static Logger					logger = Logger.getLogger(TargetViewFactory.class);

	/**
	 * Factory object for {@link lcsb.mapviewer.services.view.AnnotationView}
	 * elements.
	 */
	@Autowired
	private AnnotationViewFactory	annotationViewFactory;

	/**
	 * Service accessing
	 * <a href="http://europepmc.org/RestfulWebService">pubmed</a>.
	 */
	@Autowired
	private PubmedParser					pubmedParser;

	@Override
	public TargetView create(Target target) {
		return create(target, null);
	}

	/**
	 * Creates {@link TargetView} element for given {@link Target} and places
	 * links on the parameter {@link Model}.
	 * 
	 * @param target
	 *          object for which {@link TargetView} is created
	 * @param model
	 *          {@link Model} where links will be placed
	 * @return {@link TargetView} element for given {@link Target}
	 */
	public TargetView create(Target target, Model model) {
		TargetView result = new TargetView();
		if (target == null) {
			return result;
		}
		result.setName(target.getName());
		if (target.getType() == null) {
			logger.warn("Unknown type of target: " + target.getName() + ", target name: " + target.getName());
		} else {
			result.setType(target.getType().getCommonName());
		}

		if (target.getType() == TargetType.SINGLE_PROTEIN || target.getType() == TargetType.COMPLEX_PROTEIN || target.getType() == TargetType.PROTEIN_FAMILY) {
			result.setVisible(true);
			result.setSelectable(false);
		}

		for (MiriamData md : target.getGenes()) {
			GeneRow geneRow = new GeneRow();
			geneRow.setAnnotation(annotationViewFactory.create(md));
			if (model != null) {
				Set<AnnotatedObject> list = new HashSet<>();
				list.addAll(model.getElementsByName(md.getResource()));
				list.addAll(model.getElementsByAnnotation(md));

				int countInTopModel = list.size();
				for (ModelSubmodelConnection submodel : model.getSubmodelConnections()) {
					list.addAll(submodel.getSubmodel().getModel().getElementsByName(md.getResource()));
					list.addAll(submodel.getSubmodel().getModel().getElementsByAnnotation(md));
				}
				int counter = list.size();
				geneRow.setSelectable(counter > 0);

				if (counter > 0) {
					geneRow.setSelectable(true);
					if (target.getType() == TargetType.SINGLE_PROTEIN || target.getType() == TargetType.PROTEIN_FAMILY) {
						result.setSelectable(true);
					}
					if (countInTopModel < counter) {
						result.setAvailableInSubmodel(true);
					}
				}
			}
			result.addProtein(geneRow);
		}

		// this should be fixed
		// if (target.getType() == TargetType.COMPLEX_PROTEIN) {
		//
		// if (complexToOverlayList(target, null).size() > 0) {
		// row.setSelectable(true);
		// }
		// }

		if (target.getSource() != null) {
			result.setAnnotation(annotationViewFactory.create(target.getSource()));
		}
		for (MiriamData md : target.getReferences()) {
			if (md.getDataType().equals(MiriamType.PUBMED)) {
				try {
					Integer id = Integer.parseInt(md.getResource());
					Article reference = pubmedParser.getPubmedArticleById(id);
					if (reference != null) {
						result.addReference(reference);
					}
				} catch (NumberFormatException e) {
					logger.warn("Invalid pubmed reference: " + md);
				} catch (PubmedSearchException e) {
					logger.warn("Problem with accessing info about pubmed: " + md, e);
				}
			} else {
				logger.warn("Unknown reference type: " + md);
			}
		}
		Collections.sort(result.getReferences(), new ArticleCitationComparator());

		return result;

	}

	@Override
	public String createGson(TargetView object) {
		return new Gson().toJson(object);
	}

	@Override
	public Target viewToObject(TargetView view) {
		throw new NotImplementedException();
	}
}
