package lcsb.mapviewer.services.search.db.chemical;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;

import lcsb.mapviewer.annotation.data.Article;
import lcsb.mapviewer.annotation.data.Chemical;
import lcsb.mapviewer.common.comparator.StringComparator;
import lcsb.mapviewer.services.search.ISearchResultView;
import lcsb.mapviewer.services.search.db.TargetView;
import lcsb.mapviewer.services.view.AbstractView;
import lcsb.mapviewer.services.view.AnnotationView;

/**
 * Data of the chemical from external database to be visualized in the client
 * side.
 * 
 * @author Ayan Rota
 * 
 */
public class ChemicalView extends AbstractView<Chemical> implements ISearchResultView, Serializable {

	/**
	 * 
	 */
	private static final long		 serialVersionUID		= 91486136745430139L;

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger				 logger							= Logger.getLogger(ChemicalView.class);

	/**
	 * Is the chemical selected by the client or not.
	 */
	private boolean							 selected						= false;

	/**
	 * Name of the chemical.
	 */
	private String							 name;

	/**
	 * Detailed description of the MeSH object.
	 */
	private String							 description;

	/**
	 * List of synonyms.
	 */
	private String							 synonyms;

	/**
	 * Link for cas id.
	 */
	private AnnotationView			 casLink;

	/**
	 * The direct Evedince if any.
	 */
	private String							 directEvidence;

	/**
	 * List of links to extenral databases.
	 */
	private List<Article>				 directEvidencePubs	= new ArrayList<>();

	/**
	 * Link for source.
	 */
	private AnnotationView			 sourceLink;

	/**
	 * List of targets for the chemical.
	 */
	private List<TargetView> targetRows					= new ArrayList<>();

	/***
	 * Constructor that initialize the data with information from the
	 * parameter*object.
	 ** 
	 * @param chemical
	 *          original Chemical from which the data will be initialized
	 */

	protected ChemicalView(Chemical chemical) {
		// chemical is not db entity so don't pass it to super constructor
		super(null);
	}

	/**
	 * Default constructor. Should be used only for deserialization.
	 */
	protected ChemicalView() {
	}

	/**
	 * Default constructor that creates a copy.
	 * 
	 * @param original
	 *          origianl object that should be copied
	 */
	public ChemicalView(ChemicalView original) {
		super(null);
		selected = original.isSelected();
		name = original.getName();
		targetRows.addAll(original.getTargetRows());
	}

	/**
	 * Comparator of the objects by their name.
	 * 
	 * @author Piotr Gawron
	 * 
	 */
	public static class NameComparator implements Comparator<ChemicalView> {
		/**
		 * Default string comparator.
		 */
		private StringComparator stringComparator = new StringComparator();

		@Override
		public int compare(ChemicalView arg0, ChemicalView arg1) {
			return stringComparator.compare(arg0.getName(), arg1.getName());
		}

	}

	/**
	 * @return the selected
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * @param selected
	 *          the selected to set
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the directEvedince
	 */
	public String getDirectEvedince() {
		return directEvidence;
	}

	/**
	 * @param directEvedince
	 *          the directEvedince to set
	 */
	public void setDirectEvedince(String directEvedince) {
		this.directEvidence = directEvedince;
	}

	/**
	 * @return the targetRows
	 */
	public List<TargetView> getTargetRows() {
		return targetRows;
	}

	/**
	 * @param targetRows
	 *          the targetRows to set
	 */
	public void setTargetRows(List<TargetView> targetRows) {
		this.targetRows = targetRows;
	}

	/**
	 * Adds object to {@link #directEvidencePubs}.
	 * 
	 * @param article
	 *          object to add
	 */
	public void addDirectEvidence(Article article) {
		directEvidencePubs.add(article);
	}

	/**
	 * @return casLink
	 */
	public AnnotationView getCasLink() {
		return casLink;
	}

	/**
	 * @param casLink
	 *          a link to Cas
	 */
	public void setCasLink(AnnotationView casLink) {
		this.casLink = casLink;
	}

	/**
	 * @return direct evidence.
	 */
	public String getDirectEvidence() {
		return directEvidence;
	}

	/**
	 * @param directEvidence
	 *          a text to indicate if there is direct evidence found.
	 */
	public void setDirectEvidence(String directEvidence) {
		this.directEvidence = directEvidence;
	}

	/**
	 * @return the link to the source database of chemicals.
	 */
	public AnnotationView getSourceLink() {
		return sourceLink;
	}

	/**
	 * @param sourceLink
	 *          the link to database of chemicals.
	 */
	public void setSourceLink(AnnotationView sourceLink) {
		this.sourceLink = sourceLink;
	}

	/**
	 * Returns link to pubmed that enlist all references.
	 * 
	 * @return link to pubmed that enlist all references
	 */
	public String getAllDirectEvidenceLinks() {
		String result = "http://www.ncbi.nlm.nih.gov/pubmed/?term=";
		for (Article publication : directEvidencePubs) {
			result += publication.getId() + " ";
		}
		return result;
	}

	/**
	 * @return all articles/publications associated with the chemical direct
	 *         evidence.
	 */
	public List<Article> getDirectEvidencePubs() {
		return directEvidencePubs;
	}

	/**
	 * @param directEvidencePubs
	 *          all articles/publications associated with the chemical direct
	 *          evidence.
	 */
	public void setDirectEvidencePubs(List<Article> directEvidencePubs) {
		this.directEvidencePubs = directEvidencePubs;
	}

	/**
	 * @param description
	 *          description of the Chemicals.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return description of Chemical.
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @return list of Synonyms.
	 */
	public String getSynonyms() {
		return synonyms;
	}

	/**
	 * @param synonyms
	 *          list of Synonyms.
	 */
	public void setSynonyms(String synonyms) {
		this.synonyms = synonyms;
	}

	@Override
	public String getUniqueId() {
		return name;
	}

	@Override
	public void setUniqueId(String uniqueId) {
		this.name = uniqueId;
	}

}
