package lcsb.mapviewer.services.search.db.chemical;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.annotation.data.Article;
import lcsb.mapviewer.annotation.data.ArticleCitationComparator;
import lcsb.mapviewer.annotation.data.Chemical;
import lcsb.mapviewer.annotation.data.MeSH;
import lcsb.mapviewer.annotation.data.Target;
import lcsb.mapviewer.annotation.services.MeSHParser;
import lcsb.mapviewer.annotation.services.PubmedParser;
import lcsb.mapviewer.annotation.services.PubmedSearchException;
import lcsb.mapviewer.annotation.services.annotators.AnnotatorException;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.overlay.IconManager;
import lcsb.mapviewer.services.overlay.IconType;
import lcsb.mapviewer.services.search.SearchResultFactory;
import lcsb.mapviewer.services.search.data.ElementIdentifier;
import lcsb.mapviewer.services.search.db.DrugTargetViewVisibilityComparator;
import lcsb.mapviewer.services.search.db.TargetView;
import lcsb.mapviewer.services.search.db.TargetViewFactory;
import lcsb.mapviewer.services.view.AnnotationViewFactory;

/**
 * Factory class for {@link ChemicalView} class.
 * 
 * @author Ayan Rota
 * 
 */
public class ChemicalViewFactory extends SearchResultFactory<Chemical, ChemicalView> {

	/**
	 * Default class logger.
	 */
	private static Logger					logger = Logger.getLogger(ChemicalViewFactory.class);

	/**
	 * Factory object used for creation of
	 * {@link lcsb.mapviewer.services.view.AnnotationView} elements.
	 */
	@Autowired
	private AnnotationViewFactory	annotationViewFactory;

	/**
	 * Service accessing
	 * <a href="http://europepmc.org/RestfulWebService">pubmed</a>.
	 */
	@Autowired
	private PubmedParser					pubmedParser;

	/**
	 * Factory object used for creation of {@link TargetView} elements.
	 */
	@Autowired
	private TargetViewFactory			drugTargetViewFactory;

	/**
	 * Service accessing <a href="http://europepmc.org/RestfulWebService">mesh</a>
	 * .
	 */
	@Autowired
	private MeSHParser						meSHParser;

	@Override
	public ChemicalView create(Chemical chemical) {
		return create(chemical, null, 0);
	}

	/**
	 * Creates {@link ChemicalView}.
	 * 
	 * @param chemical
	 *          original Chemical from which the data will be initialized
	 * @param model
	 *          model where the view will be presented
	 * @param set
	 *          which set of icons should be used
	 * 
	 * @return {@link ChemicalView} object for given chemical. links are placed
	 *         according to model given in the parameter.
	 */
	public ChemicalView create(Chemical chemical, Model model, int set) {
		ChemicalView chemicalView = new ChemicalView(chemical);
		if (chemical == null) {
			return chemicalView;
		}

		chemicalView.setName(chemical.getChemicalName());
		if (chemical.getDirectEvidence() != null) {
			chemicalView.setDirectEvedince(chemical.getDirectEvidence().getValue());
		}

		if (chemical.getChemicalId() != null) {
			chemicalView.setSourceLink(annotationViewFactory.create(chemical.getChemicalId()));
			try {
				MeSH mesh = meSHParser.getMeSH(chemical.getChemicalId());
				if (mesh != null) {
					chemicalView.setDescription(mesh.getDescription());
					chemicalView.setSynonyms(mesh.getSynonymsString());
				} else {
					chemicalView.setDescription("Mesh term not available");
					chemicalView.setSynonyms("N/A");
				}
			} catch (AnnotatorException e) {
				logger.error("Problem with accessing mesh database", e);
				chemicalView.setDescription("Mesh term not available");
				chemicalView.setSynonyms("N/A");
			}
		}

		if (chemical.getCasID() != null) {
			chemicalView.setCasLink(annotationViewFactory.create(chemical.getCasID()));
		}

		List<Article> articles = new ArrayList<Article>();
		for (MiriamData publication : chemical.getDirectEvidencePublication()) {
			if (publication.getDataType().equals(MiriamType.PUBMED)) {
				Integer id = Integer.parseInt(publication.getResource());
				try {
					Article article = pubmedParser.getPubmedArticleById(id);
					if (article != null) {
						articles.add(article);
					}
				} catch (PubmedSearchException e) {
					logger.error("Problem with accessing info about pubmed", e);
				}
			} else {
				logger.warn("Unknown publication type: " + publication);
			}
		}
		Collections.sort(articles, new ArticleCitationComparator());
		chemicalView.setDirectEvidencePubs(articles);

		List<TargetView> targetsRows = new ArrayList<>();

		int differentNames = 0;
		for (Target geneEntry : chemical.getInferenceNetwork()) {
			targetsRows.add(drugTargetViewFactory.create(geneEntry, model));
		}

		Collections.sort(targetsRows, new DrugTargetViewVisibilityComparator());

		for (TargetView geneTargetView : targetsRows) {
			if (geneTargetView.getSelectable()) {
				String icon = IconManager.getInstance().getIconForIndex(differentNames++, IconType.CHEMICAL, set);
				geneTargetView.setIcon(icon);
				geneTargetView.setSelected(true);
			}
		}
		chemicalView.setTargetRows(targetsRows);
		return chemicalView;

	}

	@Override
	public String createGson(ChemicalView object) {
		return new Gson().toJson(object);
	}

	@Override
	public List<ElementIdentifier> searchResultToElementIdentifier(ChemicalView chemical, Model inputModel) {
		List<ElementIdentifier> result = new ArrayList<>();

		List<Model> models = new ArrayList<>();
		models.addAll(inputModel.getSubmodels());
		models.add(inputModel);
		for (Model model : models) {
			for (TargetView target : chemical.getTargetRows()) {
				if (target.getSelected() && target.getIcon() != null) {
					for (Element element : model.getElements()) {
						if (elementMatch(target, element)) {
							result.add(new ElementIdentifier(element, target.getIcon()));
						}
					}
				}
			}
		}
		return result;
	}
}
