package lcsb.mapviewer.services.search.db.drug;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.annotation.data.Drug;
import lcsb.mapviewer.annotation.data.Target;
import lcsb.mapviewer.annotation.services.ChEMBLParser;
import lcsb.mapviewer.annotation.services.DrugSearchException;
import lcsb.mapviewer.annotation.services.DrugbankHTMLParser;
import lcsb.mapviewer.annotation.services.PubmedParser;
import lcsb.mapviewer.annotation.services.annotators.AnnotatorException;
import lcsb.mapviewer.annotation.services.annotators.HgncAnnotator;
import lcsb.mapviewer.common.IProgressUpdater;
import lcsb.mapviewer.model.map.AnnotatedObject;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.model.map.species.Gene;
import lcsb.mapviewer.model.map.species.Protein;
import lcsb.mapviewer.model.map.statistics.SearchType;
import lcsb.mapviewer.services.interfaces.ISearchHistoryService;
import lcsb.mapviewer.services.search.db.DbSearchCriteria;
import lcsb.mapviewer.services.view.AnnotationView;

/**
 * Implementation of the service that allows access information about drugs.
 * 
 * @author Piotr Gawron
 * 
 */
@Transactional(value = "txManager")
public class DrugService implements IDrugService {

	/**
	 * Default class logger.
	 */
	private Logger								logger = Logger.getLogger(DrugService.class);

	/**
	 * Access point and parser for the online drugbank database.
	 */
	@Autowired
	private DrugbankHTMLParser		drugBankParser;

	/**
	 * Access point and parser for the online ChEMBL database.
	 */
	@Autowired
	private ChEMBLParser					chEMBLParser;

	/**
	 * Object used to get information about {@link MiriamType#HGNC} annotations
	 * and transformations.
	 */
	@Autowired
	private HgncAnnotator					hgncAnnotator;

	/**
	 * Service accessing
	 * <a href="http://europepmc.org/RestfulWebService">pubmed</a>.
	 */
	@Autowired
	private PubmedParser					pubmedParser;

	/**
	 * Factory object used for creation of {@link DrugView} elements.
	 */
	@Autowired
	private DrugViewFactory				drugViewFactory;

	/**
	 * Service that manages search history.
	 */
	@Autowired
	private ISearchHistoryService	searchHistoryService;

	@Override
	public List<DrugView> getByNames(List<String> names, DbSearchCriteria searchCriteria) {
		List<DrugView> result = new ArrayList<>();

		Set<String> processedDrugIds = new HashSet<>();

		int oldSet = searchCriteria.getColorSet();
		for (String string : names) {
			searchCriteria.colorSet(searchCriteria.getColorSet() + 1);
			DrugView drug = getByName(string, searchCriteria);
			if (drug != null) {
				boolean isNewDrug = false;
				for (AnnotationView id : drug.getDrugLinks()) {
					if (!processedDrugIds.contains(id.toString())) {
						isNewDrug = true;
					}
					processedDrugIds.add(id.toString());
				}
				if (isNewDrug) {
					result.add(drug);
				}
			}
		}
		searchCriteria.colorSet(oldSet);

		return result;
	}

	/**
	 * Removes targets for unknown orginisms from the drug.
	 * 
	 * @param drug
	 *          drug from which we want to remove targets
	 * @param organisms
	 *          organisms that should be kept
	 */
	private void removeUnknownOrganisms(Drug drug, List<MiriamData> organisms) {
		if (organisms.size() > 0) {
			List<Target> toRemove = new ArrayList<Target>();
			for (Target target : drug.getTargets()) {
				boolean remove = true;
				for (MiriamData organism : organisms) {
					if (target.getOrganism() == null) {
						remove = false;
					} else if (target.getOrganism().equals(organism)) {
						remove = false;
					}
				}
				if (remove) {
					logger.debug("Target " + drug.getName() + " removed from list because results are limited to organisms: " + organisms);
					toRemove.add(target);
				}
			}
			drug.getTargets().removeAll(toRemove);
		}
	}

	/**
	 * Merges two drugs (typically from different databases) into one
	 * representation.
	 * 
	 * @param drug
	 *          first drug to merge
	 * @param drug2
	 *          second drug to merge
	 * @return merged drug
	 */
	private Drug mergeDrugs(Drug drug, Drug drug2) {
		if (drug == null) {
			return drug2;
		}
		if (drug2 == null) {
			return drug;
		}

		Drug result = new Drug(drug);
		result.getSources().addAll(drug2.getSources());
		if (result.getName() == null || result.getName().trim().equals("")) {
			result.setName(drug2.getName());
		}
		if (result.getDescription() == null || result.getDescription().trim().equals("")) {
			result.setDescription(drug2.getDescription());
		}
		for (String synonym : drug2.getSynonyms()) {
			if (!result.getSynonyms().contains(synonym)) {
				result.addSynonym(synonym);
			}
		}
		for (String brandName : drug2.getBrandNames()) {
			if (!result.getBrandNames().contains(brandName)) {
				result.addBrandName(brandName);
			}
		}

		if (result.getBloodBrainBarrier().equals("YES")) {
			if (drug2.getBloodBrainBarrier().equals("NO")) {
				logger.warn(
						"Problem with merging drugs. Blood Brain Barrier contains opposite values: " + result.getBloodBrainBarrier() + ", "
								+ drug2.getBloodBrainBarrier());
				result.setBloodBrainBarrier("N/A");
			}
		} else if (result.getBloodBrainBarrier().equals("NO")) {
			if (drug2.getBloodBrainBarrier().equals("YES")) {
				logger.warn(
						"Problem with merging drugs. Blood Brain Barrier contains opposite values: " + result.getBloodBrainBarrier() + ", "
								+ drug2.getBloodBrainBarrier());
				result.setBloodBrainBarrier("N/A");
			}
		} else if (result.getBloodBrainBarrier().equals("N/A")) {
			result.setBloodBrainBarrier(drug.getBloodBrainBarrier());
		} else {
			logger.warn("Unknown status of drug BBB:" + result.getBloodBrainBarrier());
		}

		result.getTargets().addAll(drug2.getTargets());
		return result;
	}

	@Override
	public DrugView getByName(String name, DbSearchCriteria searchCriteria) {
		if (name.trim().equals("")) {
			return null;
		}
		Drug drug = null;
		try {
			drug = drugBankParser.findDrug(name);
			if (drug != null) {
				removeUnknownOrganisms(drug, searchCriteria.getOrganisms());
			}
		} catch (DrugSearchException e) {
			logger.error("Problem with accessing drugBank parser", e);
		}
		try {
			Set<String> foundDrugs = new HashSet<String>();
			Set<String> searchNames = new HashSet<String>();
			// two step searching in Chembl:
			// * search for name
			searchNames.add(name);
			if (drug != null && drug.getName() != null) {
				// * search for name of a drug in drugbanke
				searchNames.add(drug.getName());
				// * search for all synonyms found in drugbank
				// searchNames.addAll(drug.getSynonyms());
			}
			for (String string : searchNames) {
				Drug drug2 = chEMBLParser.findDrug(string);
				if (drug2 != null) {
					removeUnknownOrganisms(drug2, searchCriteria.getOrganisms());
					// don't add drugs that were already found
					if (!foundDrugs.contains(drug2.getSources().get(0).getResource())) {
						drug = mergeDrugs(drug, drug2);
						foundDrugs.add(drug2.getSources().get(0).getResource());
					}
				}
			}
		} catch (DrugSearchException e) {
			logger.error("Problem with accessing ChEMBL parser", e);
		}
		if (searchCriteria.getIpAddress() != null && searchCriteria.getModel() != null) {
			searchHistoryService.addQuery(name, SearchType.GENERAL, searchCriteria.getIpAddress(), searchCriteria.getModel().getProject().getProjectId());
		}

		if (drug == null) {
			return null;
		} else {
			return drugViewFactory.create(drug, searchCriteria.getModel(), searchCriteria.getColorSet());
		}
	}

	@Override
	public List<DrugView> getForTargets(Collection<Element> targets, DbSearchCriteria searchCriteria) {
		List<Drug> drugList = new ArrayList<Drug>();
		Set<MiriamData> targetsMiriam = new HashSet<MiriamData>();
		for (Element target : targets) {
			if (target instanceof Protein || target instanceof Gene) {
				boolean hgncFound = false;
				for (MiriamData md : target.getMiriamData()) {
					if (MiriamType.HGNC_SYMBOL.equals(md.getDataType())) {
						targetsMiriam.add(md);
						hgncFound = true;
					}
				}
				if (!hgncFound) {
					MiriamData md = new MiriamData(MiriamType.HGNC_SYMBOL, target.getName());
					try {
						if (hgncAnnotator.isValidHgncMiriam(md)) {
							targetsMiriam.add(md);
						}
					} catch (AnnotatorException e) {
						logger.error("Problem with accessing HGNC database", e);
					}
				}
			}
		}
		try {
			List<Drug> drugs = drugBankParser.getDrugListByTargets(targetsMiriam);
			drugList.addAll(drugs);
		} catch (DrugSearchException e) {
			logger.error("Problem with accessing drugBank parser", e);
		}

		try {
			List<Drug> drugs = chEMBLParser.getDrugListByTargets(targetsMiriam);
			drugList.addAll(drugs);
		} catch (DrugSearchException e) {
			logger.error("Problem with accessing chembl parser", e);
		}

		for (Drug drug : drugList) {
			removeUnknownOrganisms(drug, searchCriteria.getOrganisms());
		}
		List<DrugView> result = new ArrayList<DrugView>();
		for (Drug drug : drugList) {
			result.add(drugViewFactory.create(drug));
		}

		Collections.sort(result, new DrugView.NameComparator());
		return result;
	}

	/**
	 * @return the drugBankParser
	 */
	public DrugbankHTMLParser getDrugBankParser() {
		return drugBankParser;
	}

	/**
	 * @param drugBankParser
	 *          the drugBankParser to set
	 */
	public void setDrugBankParser(DrugbankHTMLParser drugBankParser) {
		this.drugBankParser = drugBankParser;
	}

	/**
	 * @return the chEMBLParser
	 */
	public ChEMBLParser getChEMBLParser() {
		return chEMBLParser;
	}

	/**
	 * @param chEMBLParser
	 *          the chEMBLParser to set
	 */
	public void setChEMBLParser(ChEMBLParser chEMBLParser) {
		this.chEMBLParser = chEMBLParser;
	}

	/**
	 * @return the searchHistoryService
	 */
	public ISearchHistoryService getSearchHistoryService() {
		return searchHistoryService;
	}

	/**
	 * @param searchHistoryService
	 *          the searchHistoryService to set
	 */
	public void setSearchHistoryService(ISearchHistoryService searchHistoryService) {
		this.searchHistoryService = searchHistoryService;
	}

	/**
	 * @return the pubmedParser
	 * @see #pubmedParser
	 */
	public PubmedParser getPubmedParser() {
		return pubmedParser;
	}

	/**
	 * @param pubmedParser
	 *          the pubmedParser to set
	 * @see #pubmedParser
	 */
	public void setPubmedParser(PubmedParser pubmedParser) {
		this.pubmedParser = pubmedParser;
	}

	@Override
	public DrugView createEmptyView() {
		return drugViewFactory.create(null);
	}

	@Override
	public void cacheDataForModel(Model originalModel, IProgressUpdater iProgressUpdater) {
		logger.debug("Caching drug queries...");
		double progress = 0.0;
		iProgressUpdater.setProgress(progress * IProgressUpdater.MAX_PROGRESS);
		MiriamData disease = originalModel.getProject().getDisease();
		if (disease != null) {
			Set<MiriamData> targetMiriams = new HashSet<>();
			List<Model> models = new ArrayList<>();
			models.add(originalModel);
			models.addAll(originalModel.getSubmodels());
			for (Model model : models) {
				for (AnnotatedObject element : model.getAnnotatedObjects()) {
					for (MiriamData md : element.getMiriamData()) {
						if (MiriamType.HGNC_SYMBOL.equals(md.getDataType())) {
							targetMiriams.add(md);
						}
					}
				}
			}
			double counter = 0.0;
			for (MiriamData md : targetMiriams) {
				try {
					List<Drug> chemicalList = drugBankParser.getDrugListByTarget(md);
					for (Drug chemical : chemicalList) {
						drugViewFactory.create(chemical);
					}
				} catch (DrugSearchException e) {
					logger.error("Problem with accessing info about drugbank for target", e);
				}
				try {
					List<Drug> chemicalList = chEMBLParser.getDrugListByTarget(md);
					for (Drug chemical : chemicalList) {
						drugViewFactory.create(chemical);
					}
				} catch (DrugSearchException e) {
					logger.error("Problem with accessing info about chembl for target", e);
				}
				counter += 1;
				progress = counter / (double) targetMiriams.size();
				iProgressUpdater.setProgress(progress * IProgressUpdater.MAX_PROGRESS);
			}
		}
	}

}
