package lcsb.mapviewer.services.search.db.drug;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.annotation.data.Drug;
import lcsb.mapviewer.annotation.data.Target;
import lcsb.mapviewer.annotation.data.TargetType;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.overlay.IconManager;
import lcsb.mapviewer.services.overlay.IconType;
import lcsb.mapviewer.services.search.SearchResultFactory;
import lcsb.mapviewer.services.search.data.ElementIdentifier;
import lcsb.mapviewer.services.search.db.DrugTargetViewVisibilityComparator;
import lcsb.mapviewer.services.search.db.GeneRow;
import lcsb.mapviewer.services.search.db.TargetView;
import lcsb.mapviewer.services.search.db.TargetViewFactory;
import lcsb.mapviewer.services.view.AnnotationView;

/**
 * Factory class for {@link DrugView} class.
 * 
 * @author Piotr Gawron
 * 
 */
public class DrugViewFactory extends SearchResultFactory<Drug, DrugView> {
	/**
	 * Default class logger.
	 */
	private static Logger			logger = Logger.getLogger(DrugViewFactory.class);

	/**
	 * Factory object used for creation of {@link TargetView} elements.
	 */
	@Autowired
	private TargetViewFactory	targetViewFactory;

	@Override
	public DrugView create(Drug drug) {
		return create(drug, null, 0);
	}

	/**
	 * Creates {@link DrugView}.
	 * 
	 * @param drug
	 *          original Drug from which the data will be initialized
	 * @param model
	 *          model where the view will be presented
	 * @param set
	 *          which set of icons should be used
	 * 
	 * @return {@link DrugView} object for given drug. links are placed according
	 *         to model given in the parameter.
	 */
	public DrugView create(Drug drug, Model model, int set) {
		DrugView result = new DrugView(drug);
		if (drug == null) {
			return result;
		}
		result.setName(drug.getName());
		for (MiriamData md : drug.getSources()) {
			AnnotationView view = getAnnotationViewFactory().create(md);
			result.addDrugLink(view);

		}
		result.setDescription(drug.getDescription());
		result.setBloodBrainBarrier(drug.getBloodBrainBarrier());
		result.addBrandNames(drug.getBrandNames());
		result.addSynonyms(drug.getSynonyms());

		int differentNames = 0;

		for (Target target : drug.getTargets()) {
			result.addTargetRow(targetViewFactory.create(target, model));
		}
		Collections.sort(result.getTargetRows(), new DrugTargetViewVisibilityComparator());
		for (TargetView row : result.getTargetRows()) {
			if (row.getVisible()) {
				String icon;
				icon = IconManager.getInstance().getIconForIndex(differentNames++, IconType.DRUG, set);
				row.setIcon(icon);
				if (row.getSelectable()) {
					row.setSelected(true);
				}
			}
		}

		Map<String, Integer> dbEntries = new HashMap<String, Integer>();
		for (AnnotationView dl : result.getDrugLinks()) {
			Integer counter = dbEntries.get(dl.getName());
			if (counter == null) {
				counter = 1;
			} else {
				counter += 1;
			}
			dbEntries.put(dl.getName(), counter);
		}
		for (Entry<String, Integer> entry : dbEntries.entrySet()) {
			if (entry.getValue() == 1) {
				entry.setValue(null);
			} else {
				entry.setValue(0);
			}
		}
		for (AnnotationView dl : result.getDrugLinks()) {
			Integer counter = dbEntries.get(dl.getName());
			if (counter != null) {
				counter += 1;
				dbEntries.put(dl.getName(), counter);
				dl.setName(dl.getName() + " (" + counter + ")");
			}
		}
		return result;
	}

	@Override
	public String createGson(DrugView object) {
		return new Gson().toJson(object);
	}

	/**
	 * Creates list of markers for the given geneName.
	 * 
	 * @param annotation
	 *          annotation of a gene (hgnc)
	 * @param icon
	 *          what icon should be used by markers
	 * @param models
	 *          list of models where we want to create markers
	 * @return list of markers to be presented on the client side for the given
	 *         geneName
	 */
	protected List<ElementIdentifier> geneToOverlayList(AnnotationView annotation, String icon, List<Model> models) {
		List<ElementIdentifier> result = new ArrayList<>();

		for (Model m : models) {
			List<Element> species = m.getElementsByName(annotation.getName());
			for (Element species2 : species) {
				result.add(new ElementIdentifier(species2, icon));
			}
		}
		return result;
	}

	/**
	 * Creates list of markers for the given chembl target.
	 * 
	 * @param complexTarget
	 *          chembl target
	 * @param models
	 *          list of models where we want to create markers
	 * @return list of markers to be presented on the client side for the given
	 *         chembl target
	 */
	protected List<ElementIdentifier> complexToOverlayList(TargetView complexTarget, Collection<Model> models) {
		List<ElementIdentifier> result = new ArrayList<>();

		for (Model m : models) {
			for (Element element : m.getElements()) {
				if (elementMatch(complexTarget, element)) {
					result.add(new ElementIdentifier(element, complexTarget.getIcon()));
				}
			}
		}
		return result;
	}

	@Override
	public List<ElementIdentifier> searchResultToElementIdentifier(DrugView drug, Model model) {
		List<ElementIdentifier> result = new ArrayList<>();

		List<Model> models = new ArrayList<>();
		models.add(model);
		models.addAll(model.getSubmodels());

		// to be more specific add markers for targets
		for (TargetView row : drug.getTargetRows()) {
			// check if target should be vizualized
			if (row.getSelected()) {
				// depending on the type vizualize:
				if (row.getType().equals(TargetType.COMPLEX_PROTEIN.getCommonName())) {
					// complex children
					result.addAll(complexToOverlayList(row, models));
				} else if (row.getType().equals(TargetType.SINGLE_PROTEIN.getCommonName())) {
					// single protein
					if (row.getProteins().size() > 0) {
						result.addAll(geneToOverlayList(row.getProteins().get(0).getAnnotation(), row.getIcon(), models));
					}
				} else if (row.getType().equals(TargetType.PROTEIN_FAMILY.getCommonName())) {
					// all proteins in the family
					for (GeneRow geneRow : row.getProteins()) {
						result.addAll(geneToOverlayList(geneRow.getAnnotation(), row.getIcon(), models));
					}
				} else {
					logger.error("Unknown target type: " + row.getType());
				}
			} else { // if the whole target wasn't selected then maybe only some
							 // proteins within it were selected
				for (GeneRow geneRow : row.getProteins()) {
					if (geneRow.getSelected()) {
						result.addAll(geneToOverlayList(geneRow.getAnnotation(), row.getIcon(), models));
					}
				}
			}
		}

		return result;
	}

}
