package lcsb.mapviewer.services.utils.gmap;

import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelData;

import org.apache.log4j.Logger;
import org.primefaces.model.map.LatLng;

/**
 * This class is a converter of coordinates from/to gmap coordinates. Class is
 * immutable and defined for specific {@link Model}.
 * 
 * @author Piotr Gawron
 * 
 */
public final class CoordinationConverter implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Defines how big zoom level factor should be considered when computing
	 * search distance on different levels. The value should be in <1.0 - 2.0>
	 * range. 1.0 means that when level changes distance will stay the same on all
	 * levels. 2.0 means that distance in pixels will remain the same.
	 */
	private static final double ZOOM_FACTOR_PER_LEVEL = 1.5;

	/**
	 * Full angle in degrees.
	 */
	private static final int		FULL_ANGLE_IN_DEGREES = 360;

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger			 logger								= Logger.getLogger(CoordinationConverter.class);

	/**
	 * Take a look at {@link toPoint()}.
	 */

	private static final double MAX_SIN_VAL					 = 0.9999;

	/**
	 * What is the zoom factor on the lowest level.
	 */
	private final double				zoomFactor;
	/**
	 * What is the max zoom level (in our terminology the lowest level).
	 */
	private final int					 maxZoomLevel;
	/**
	 * Size of the tile used by map API.
	 */
	private final double				tileSize;
	/**
	 * How many pixels are used for longitude degree.
	 */
	private final double				pixelsPerLonDegree;
	/**
	 * How many pixels are used for longitude radian.
	 */
	private final double				pixelsPerLonRadian;
	/**
	 * This should define the origin of the map in the google maps API. It's
	 * strange but it works :).
	 */
	private final Point2D			 pixelOrigin;

	/**
	 * Bounds value in the optMin, optMax range.
	 * 
	 * @param value
	 *          value to be bounded
	 * @param optMin
	 *          minimum acceptable value
	 * @param optMax
	 *          maximum acceptable value
	 * @return value closest to inital but in optMin, optMax range
	 */
	private double bound(double value, double optMin, double optMax) {
		double result = Math.max(value, optMin);
		result = Math.min(result, optMax);
		return result;
	}

	/**
	 * Default constructor. Initializes all converter parameters for the given
	 * model.
	 * 
	 * @param model
	 *          {@link Model} for which converter is created
	 */
	public CoordinationConverter(Model model) {
		if (model.getTileSize() == 0) {
			throw new InvalidArgumentException("Size of tile cannot be 0.");
		}
		zoomFactor = (Math.max(model.getWidth(), model.getHeight())) / ((double) (model.getTileSize()) / ((double) (1 << Configuration.MIN_ZOOM_LEVEL)));
		maxZoomLevel = model.getZoomLevels() + Configuration.MIN_ZOOM_LEVEL;
		tileSize = model.getTileSize();
		pixelOrigin = new Point2D.Double(tileSize / 2, tileSize / 2);
		pixelsPerLonDegree = tileSize / FULL_ANGLE_IN_DEGREES;
		pixelsPerLonRadian = tileSize / (2 * Math.PI);
	}

	/**
	 * Default constructor. Initializes all converter parameters for the given
	 * model.
	 * 
	 * @param submodel
	 *          {@link Model} for which converter is created
	 */
	public CoordinationConverter(ModelData submodel) {
		this(submodel.getModel());
	}

	/**
	 * Transform lat, long coordinates into x,y coordinates.
	 * 
	 * @param latLng
	 *          coordinates in lat long format
	 * @return absolute x,y coordinates
	 */
	public Point2D toPoint(LatLng latLng) {
		Point2D origin = pixelOrigin;

		double x = origin.getX() + latLng.getLng() * pixelsPerLonDegree;

		// NOTE(appleton): Truncating to 0.9999 effectively limits latitude to
		// 89.189. This is about a third of a tile past the edge of the world
		// tile.
		double siny = bound(Math.sin(Math.toRadians(latLng.getLat())), -MAX_SIN_VAL, MAX_SIN_VAL);
		double y = origin.getY() + Math.log((1 + siny) / (1 - siny)) / 2.0 * -pixelsPerLonRadian;

		x *= zoomFactor;
		y *= zoomFactor;
		return new Point2D.Double(x, y);
	};

	/**
	 * Transforms absolute x,y coordinates into lat long coordinate system.
	 * 
	 * @param param
	 *          absolute x,y coordinate
	 * @return lat long coordinate
	 */
	public LatLng toLatLng(Point2D param) {
		Point2D point = new Point2D.Double(param.getX() / zoomFactor, param.getY() / zoomFactor);
		Point2D origin = pixelOrigin;
		double lng = (point.getX() - origin.getX()) / pixelsPerLonDegree;
		double latRadians = (point.getY() - origin.getY()) / -pixelsPerLonRadian;
		double lat = Math.toDegrees(2 * Math.atan(Math.exp(latRadians)) - Math.PI / 2);
		return new LatLng(lat, lng);
	}

	/**
	 * Transforms polygon described with set of lat long coordinates into
	 * {@link Path2D} object trat represents polgyon in absolute coordinates.
	 * 
	 * @param latLngPolygon
	 *          string representing polygon in latlng format. Points are separated
	 *          by ";" character. lat from lng coordinate is separated by ",".
	 *          absolute x,y coordinate
	 * @return polygon in absolute coordinates
	 */
	public Path2D latLngToPolygon(String latLngPolygon) {
		Path2D result = new Path2D.Double();
		String[] latLngArray = latLngPolygon.split(";");

		List<Point2D> points = new ArrayList<Point2D>();
		for (String string : latLngArray) {
			if (!string.trim().equals("")) {
				double lat = Double.valueOf(string.split(",")[0]);
				double lng = Double.valueOf(string.split(",")[1]);
				if (lng > 0) {
					lng -= FULL_ANGLE_IN_DEGREES;
				}
				LatLng latLng = new LatLng(lat, lng);
				points.add(toPoint(latLng));
			}
		}
		result.moveTo(points.get(0).getX(), points.get(0).getY());
		for (int i = 1; i < points.size(); i++) {
			Point2D point = points.get(i);
			result.lineTo(point.getX(), point.getY());
		}
		result.closePath();
		return result;
	}

	/**
	 * Computes distance that should be used on different zoom levels.
	 * 
	 * @param searchDistance
	 *          distance that should be used on the deepest level
	 * @param zoomLevel
	 *          ratio that describes how much the distance should change on higher
	 *          levels
	 * @return distance that should be used on zoomLevel
	 */
	public double scale(double searchDistance, String zoomLevel) {
		int levels = 0;
		if (zoomLevel == null || zoomLevel.equals("")) {
			levels = maxZoomLevel - Configuration.MIN_ZOOM_LEVEL;
		} else {
			levels = maxZoomLevel - Integer.valueOf(zoomLevel);
		}
		return Math.pow(ZOOM_FACTOR_PER_LEVEL, levels) * searchDistance;
	};

}
