package lcsb.mapviewer.services.view;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.springframework.transaction.annotation.Transactional;

import com.google.gson.Gson;

/**
 * This abstract class represents factory that should be used to create
 * {@link AbstractView} classes that are accessible from the client side.
 * 
 * @author Piotr Gawron
 * 
 * @param <T>
 *          original class
 * @param <S>
 *          {@link AbstractView} class of the original class
 */
@Transactional(value = "txManager")
public abstract class AbstractViewFactory<T, S> {

	/**
	 * Creates view object for the given parameter.
	 * 
	 * @param object
	 *          object that should be transformed into {@link AbstractView}
	 * @return {@link AbstractView} of the object
	 */
	public abstract S create(T object);

	/**
	 * Creates original object for the given view. Typical implementation should
	 * retrieve it from db and update the information from parameter.
	 * 
	 * @param view
	 *          {@link AbstractView} of the object
	 * @return original object
	 */
	public abstract T viewToObject(S view);

	/**
	 * Creates Gson string representing {@link AbstractView<T>} element.
	 * 
	 * @param object
	 *          object taht should be transformed into a string
	 * @return string gson representation
	 */
	public abstract String createGson(S object);

	/**
	 * Creates Gson string representing list of {@link AbstractView<T>} elements.
	 * 
	 * @param list
	 *          list of objects taht should be transformed into a string
	 * @return {@link String} with gson representation
	 */
	public String createGson(List<S> list) {
		return new Gson().toJson(list);
	}

	/**
	 * Creates list of views for the given list of objects.
	 * 
	 * @param objects
	 *          list of objects that should be transformed into list of
	 *          {@link AbstractView views}
	 * @return list of {@link AbstractView views} for the given list of objects.
	 */
	public List<S> createList(Collection<T> objects) {
		List<S> result = new ArrayList<S>();
		for (T object : objects) {
			result.add(create(object));
		}
		return result;
	}
}
