package lcsb.mapviewer.services.view;

import java.io.Serializable;

import lcsb.mapviewer.model.user.BasicPrivilege;
import lcsb.mapviewer.model.user.PrivilegeType;

/**
 * This class repesents single user privilege in the way understandable by the
 * client side.
 * 
 * @author Piotr Gawron
 * 
 */
public class PrivilegeView extends AbstractView<BasicPrivilege> implements Comparable<PrivilegeView>, Serializable {

	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;

	/**
	 * What is the name of the privilege.
	 */
	private String										commonName;

	/**
	 * Does user has this privilege.
	 */
	private Boolean										selected;

	/**
	 * Type associated with this privilege.
	 */
	private PrivilegeType							type;

	/**
	 * If privilege is associated with some object (the privilege defines access
	 * level to this object) then {@link #objectId} points to database identifier
	 * of this object.
	 */
	private Integer										objectId;

	/**
	 * Access level for this privilege. Usually it's boolean {0,1} value. However,
	 * some privileges (the ones that are characterized as {@link #numeric})
	 * accept any integer values.
	 */
	private Integer										level;

	/**
	 * This field describe if privilege is:
	 * <ul>
	 * <li>numeric - privileges that takes as an input number of something,</li>
	 * <li>boolean - only logic value is acceptable.</li>
	 * </ul>
	 */
	private Boolean										numeric;

	/**
	 * Default constructor. Creates view representation from existing privilege.
	 * 
	 * @param privilege
	 *          original privilege
	 */
	protected PrivilegeView(BasicPrivilege privilege) {
		super(privilege);
	}

	/**
	 * Default constructor. Should be used only for deserialization.
	 */
	protected PrivilegeView() {
	}

	/**
	 * Constructor that creates empty privilege representation for the type given
	 * in the parameter.
	 * 
	 * @param type2
	 *          type of the privilege
	 */
	protected PrivilegeView(PrivilegeType type2) {
		super(null);
		this.commonName = type2.getCommonName();
		this.selected = false;
		this.type = type2;
		this.objectId = null;
		this.numeric = type2.isNumeric();
		this.level = 0;
	}

	@Override
	public int compareTo(PrivilegeView o) {
		if (commonName == null) {
			return -1;
		}
		if (o == null) {
			return -1;
		}
		return this.commonName.compareTo(o.getCommonName());
	}

	/**
	 * @return the commonName
	 * @see #commonName
	 */
	public String getCommonName() {
		return commonName;
	}

	/**
	 * @param commonName
	 *          the commonName to set
	 * @see #commonName
	 */
	public void setCommonName(String commonName) {
		this.commonName = commonName;
	}

	/**
	 * @return the selected
	 * @see #selected
	 */
	public Boolean getSelected() {
		return selected;
	}

	/**
	 * @param selected
	 *          the selected to set
	 * @see #selected
	 */
	public void setSelected(Boolean selected) {
		this.selected = selected;
	}

	/**
	 * @return the type
	 * @see #type
	 */
	public PrivilegeType getType() {
		return type;
	}

	/**
	 * @param type
	 *          the type to set
	 * @see #type
	 */
	public void setType(PrivilegeType type) {
		this.type = type;
	}

	/**
	 * @return the objectId
	 * @see #objectId
	 */
	public Integer getObjectId() {
		return objectId;
	}

	/**
	 * @param objectId
	 *          the objectId to set
	 * @see #objectId
	 */
	public void setObjectId(Integer objectId) {
		this.objectId = objectId;
	}

	/**
	 * @return the level
	 * @see #level
	 */
	public Integer getLevel() {
		return level;
	}

	/**
	 * @param level
	 *          the level to set
	 * @see #level
	 */
	public void setLevel(Integer level) {
		this.level = level;
	}

	/**
	 * @return the numeric
	 * @see #numeric
	 */
	public Boolean getNumeric() {
		return numeric;
	}

	/**
	 * @param numeric
	 *          the numeric to set
	 * @see #numeric
	 */
	public void setNumeric(Boolean numeric) {
		this.numeric = numeric;
	}

}
