package lcsb.mapviewer.services.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.test.annotation.Rollback;

import lcsb.mapviewer.common.IProgressUpdater;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.graph.DataMining;
import lcsb.mapviewer.model.map.graph.DataMiningType;
import lcsb.mapviewer.model.map.reaction.Product;
import lcsb.mapviewer.model.map.reaction.Reactant;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.Protein;
import lcsb.mapviewer.model.map.species.SimpleMolecule;
import lcsb.mapviewer.services.ServiceTestFunctions;

@Rollback(true)
public class DataMiningServiceTest2 extends ServiceTestFunctions {
	static Logger logger = Logger.getLogger(DataMiningServiceTest2.class);

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testParseData() throws Exception {
		try {
			Element snca = new GenericProtein("id1");
			snca.addMiriamData(new MiriamData(MiriamType.HGNC_SYMBOL, "SNCA"));
			Element park1 = new GenericProtein("id2");
			park1.addMiriamData(new MiriamData(MiriamType.HGNC_SYMBOL, "PARK1"));
			SimpleMolecule sm = new SimpleMolecule("id3");
			sm.addMiriamData(new MiriamData(MiriamType.CHEBI, "CHEBI:1234"));

			Collection<Element> nodes = new ArrayList<>();
			nodes.add(snca);
			nodes.add(park1);

			Set<DataMining> res = dataMiningService
					.parseData("testFiles/data_mining/sample_input.txt", DataMiningType.UNKNOWN, nodes, new HashSet<>(), new IProgressUpdater() {
						@Override
						public void setProgress(double progress) {
						}
					});
			assertNotNull(res);
			assertEquals(2, res.size());
			nodes.add(sm);

			res = dataMiningService
					.parseData("testFiles/data_mining/sample_input.txt", DataMiningType.UNKNOWN, nodes, new HashSet<Reaction>(), new IProgressUpdater() {
						@Override
						public void setProgress(double progress) {
						}
					});
			assertEquals(3, res.size());
			nodes.add(sm);

			for (DataMining dm : res) {
				if (dm.getElement().equals(snca)) {
					assertEquals(1, dm.getSuggestedConnections().size());
					MiriamData md = dm.getSuggestedConnections().get(0);
					assertEquals(new MiriamData(MiriamType.HGNC_SYMBOL, "BAC"), md);
					assertEquals("unknown desc", dm.getDescription());
					assertEquals(1, dm.getReferences().size());
				} else if (dm.getElement().equals(park1)) {
					assertEquals(1, dm.getSuggestedConnections().size());
					MiriamData md = dm.getSuggestedConnections().get(0);
					assertEquals(new MiriamData(MiriamType.HGNC_SYMBOL, "PARK7"), md);
					assertEquals("desc", dm.getDescription());
					assertEquals(0, dm.getReferences().size());
				} else if (dm.getElement().equals(sm)) {
					assertEquals(1, dm.getSuggestedConnections().size());
					MiriamData md = dm.getSuggestedConnections().get(0);
					assertEquals(new MiriamData(MiriamType.CHEBI, "CHEBI:123"), md);
					assertEquals("", dm.getDescription());
					assertEquals(2, dm.getReferences().size());
				} else {
					fail("Unknown dm element: " + dm.getElement());
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testDataMiningMiriamDataFormat() throws Exception {
		try {
			Protein snca = new GenericProtein("id1");
			snca.addMiriamData(new MiriamData(MiriamType.HGNC, "18618"));

			Collection<Element> nodes = new ArrayList<>();
			nodes.add(snca);

			Set<DataMining> res = dataMiningService
					.parseData("testFiles/data_mining/test_dm.txt", DataMiningType.UNKNOWN, nodes, new HashSet<Reaction>(), new IProgressUpdater() {
						@Override
						public void setProgress(double progress) {
						}
					});
			assertNotNull(res);

			for (DataMining dataMining : res) {
				for (MiriamData md : dataMining.getReferences())
					assertFalse("Invalid miriam resource: " + md.getResource(), md.getResource().contains("urn"));
			}

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testParseDataWithFiltering() throws Exception {
		try {
			Protein park7 = new GenericProtein("id1");
			park7.addMiriamData(new MiriamData(MiriamType.HGNC_SYMBOL, "PARK7"));
			Protein park1 = new GenericProtein("id2");
			park1.addMiriamData(new MiriamData(MiriamType.HGNC_SYMBOL, "PARK1"));

			Collection<Element> nodes = new ArrayList<>();
			nodes.add(park7);
			nodes.add(park1);

			Reaction reaction = new Reaction();
			reaction.addProduct(new Product(park7));
			reaction.addReactant(new Reactant(park1));

			Set<Reaction> reactions = new HashSet<Reaction>();
			reactions.add(reaction);

			Set<DataMining> res = dataMiningService
					.parseData("testFiles/data_mining/sample_input.txt", DataMiningType.UNKNOWN, nodes, reactions, new IProgressUpdater() {
						@Override
						public void setProgress(double progress) {
						}
					});
			assertNotNull(res);
			assertEquals(0, res.size());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testParseDataWithFiltering2() throws Exception {
		try {
			Protein park7 = new GenericProtein("id1");
			park7.addMiriamData(new MiriamData(MiriamType.HGNC_SYMBOL, "PARK7"));
			Protein park1 = new GenericProtein("id2");
			park1.addMiriamData(new MiriamData(MiriamType.HGNC_SYMBOL, "PARK1"));

			Collection<Element> nodes = new ArrayList<>();
			nodes.add(park7);
			nodes.add(park1);

			Reaction reaction = new Reaction();
			reaction.addProduct(new Product(park1));
			reaction.addReactant(new Reactant(park7));

			Set<Reaction> reactions = new HashSet<Reaction>();
			reactions.add(reaction);

			Set<DataMining> res = dataMiningService
					.parseData("testFiles/data_mining/sample_input.txt", DataMiningType.UNKNOWN, nodes, reactions, new IProgressUpdater() {
						@Override
						public void setProgress(double progress) {
						}
					});
			assertNotNull(res);
			assertEquals(0, res.size());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}
}
