package lcsb.mapviewer.services.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.commands.CreateHierarchyCommand;
import lcsb.mapviewer.converter.model.celldesigner.reaction.ReactionLineData;
import lcsb.mapviewer.model.map.compartment.Compartment;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.reaction.ReactionNode;
import lcsb.mapviewer.model.map.species.Complex;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.Protein;
import lcsb.mapviewer.model.map.species.Species;
import lcsb.mapviewer.services.ServiceTestFunctions;
import lcsb.mapviewer.services.interfaces.IExporterService;
import lcsb.mapviewer.services.interfaces.IExporterService.ExporterParameters;
import lcsb.mapviewer.services.utils.data.ExportColumn;
import lcsb.mapviewer.services.utils.data.ExportFileType;
import lcsb.mapviewer.services.view.PubmedAnnotatedElementsView;

public class ExporterServiceTest extends ServiceTestFunctions {
	Logger					 logger	= Logger.getLogger(ExporterServiceTest.class);
	@Autowired
	IExporterService exporter2;

	ExporterService	 exporter;

	@Before
	public void setUp() throws Exception {
		exporter = (ExporterService) exporter2;
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testGetExportSpeciesStringForTabSeparatedFile() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export_with_artifitial_comp.xml", false);
			ExportColumn columns[] = { ExportColumn.NAME, ExportColumn.ID, ExportColumn.COMPONENT_NAME, ExportColumn.COMPARTMENT_NAME, ExportColumn.TYPE };

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).column(columns).fileType(ExportFileType.TAB_SEPARATED);

			// full export (all elements)
			String string = exporter.getExportSpeciesString(params);
			String lines[] = string.split("\n");
			assertTrue("Not enough elements in the result file", lines.length > 1);

			String response = lines[0];
			String textColumns[] = response.split("\t");
			assertEquals(5, textColumns.length);

			// only proteins for self-made modules
			new CreateHierarchyCommand(model, 8, 80).execute();
			string = exporter.getExportSpeciesString(params.type(GenericProtein.class));
			lines = string.split("\n");
			assertTrue(lines.length > 1);

			boolean differenceBetweenComponentCompartmnet = false;
			int lineCount = 0;
			for (String string2 : lines) {
				if (lineCount == 0) {
					textColumns = string2.split("\t");
					assertEquals(ExportColumn.TYPE.getTitle(), textColumns[4]);
				} else {
					textColumns = string2.split("\t");
					assertEquals(GenericProtein.class.getSimpleName(), textColumns[4]);
					if (!textColumns[2].equalsIgnoreCase(textColumns[3])) {
						differenceBetweenComponentCompartmnet = true;
					}
				}
				lineCount++;
			}
			assertTrue(differenceBetweenComponentCompartmnet);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testDuplicatesInGetExportSpeciesString() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export_duplicate_elements.xml", false);
			ExportColumn columns[] = { ExportColumn.NAME, ExportColumn.COMPONENT_NAME, ExportColumn.TYPE };

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).column(columns).fileType(ExportFileType.TAB_SEPARATED);

			// full export (all elements)
			String string = exporter.getExportSpeciesString(params);

			String lines[] = string.split("\n");
			Set<String> uniqueNames = new HashSet<String>();
			for (String string2 : lines) {
				uniqueNames.add(string2);
			}
			assertEquals(uniqueNames.size(), lines.length);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetExportStringForOneSpecies_Complex() {
		try {
			Model model = getModelForFile("testFiles/export/hsp70.xml", true);

			Species alias = (Species) model.getElementByElementId("csa1");
			ExportColumn columns[] = { ExportColumn.NAME };

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).//
					column(columns).//
					complexElementsName(false).//
					fileType(ExportFileType.TAB_SEPARATED);

			List<String> elements = exporter.getExportStringForOneSpecies(alias, params);
			assertTrue(elements.contains("HSP70:chaperon"));

			// now the results are concatenated children
			params.complexElementsName(true);

			elements = exporter.getExportStringForOneSpecies(alias, params);
			assertTrue(elements.size() > 0);
			String string = elements.get(0);
			assertFalse(string.equals("HSP70:chaperon"));
			assertTrue(string.contains("HSP70"));
			assertTrue(string.contains("STUB1"));
			assertTrue(string.contains("DNAJB2"));
			assertTrue(string.contains("BAG1"));

			// and now check complexes with mix of proteins and molecules
			alias = (Species) model.getElementByElementId("csa3");

			elements = exporter.getExportStringForOneSpecies(alias, params);
			assertTrue(elements.size() > 0);
			string = elements.get(0);
			assertTrue(string.contains("UBA1"));
			assertTrue(string.contains("UBA6"));
			assertTrue(string.contains("AMP"));

			elements = exporter.getExportStringForOneSpecies(alias, params.type(Protein.class));
			assertTrue(elements.size() > 0);
			string = elements.get(0);
			assertTrue(string.contains("UBA1"));
			assertTrue(string.contains("UBA6"));
			assertFalse(string.contains("AMP"));

		} catch (Exception e) {
			e.printStackTrace();
			fail("Exception occured");
		}
	}

	@Test
	public void testGetExportStringForOneReaction() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export/reaction.xml", false);
			Set<Compartment> aliases = new HashSet<>();
			for (Compartment alias : model.getCompartments()) {
				if (alias.getName().equalsIgnoreCase("Dopamine loaded synaptic vesicle"))
					aliases.add(alias);
			}

			Reaction reaction = model.getReactionByReactionId("re2");

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).//
					fileType(ExportFileType.SIF).//
					type(Protein.class).//
					type(Complex.class).//
					fileType(ExportFileType.SIF).//
					complexElementsName(false).//
					excluded(aliases);

			String string = exporter.getExportSingleInteractionString(reaction, params).get(0);
			assertNotNull(string);
			assertTrue(string.contains("Calcium channel:RIMBP:RIM:MUNC13:RAB"));
			assertTrue(string.contains("CSP:Hsc70:SGT"));
			assertTrue(string.contains("SNCA"));
			assertTrue(string.contains("trans-SNARE complex"));
			assertFalse(string.contains("VAMP2"));

			params = new IExporterService.ExporterParameters().model(model).fileType(ExportFileType.SIF).type(Complex.class).type(Protein.class);
			string = exporter.getExportSingleInteractionString(reaction, params).get(0);
			assertTrue(string.contains("VAMP2"));

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testGetExportStringForOneReactionInGeneral() {
		try {
			Model model = getModelForFile("testFiles/export/reaction.xml", true);
			Reaction reaction = model.getReactionByReactionId("re2");
			ExporterParameters params = new IExporterService.ExporterParameters().model(model).//
					fileType(ExportFileType.SIF).//
					complexElementsName(false).//
					type(Complex.class).type(Protein.class);
			String string = exporter.getExportSingleInteractionString(reaction, params).get(0);
			assertNotNull(string);
			assertTrue(string.contains("Calcium channel:RIMBP:RIM:MUNC13:RAB"));
			assertTrue(string.contains("CSP:Hsc70:SGT"));
			assertTrue(string.contains("SNCA"));
			assertTrue(string.contains("trans-SNARE complex"));
			assertTrue(string.contains("SNAP25"));
			assertFalse(string.contains("RAB3A"));

			params.complexElementsName(true);
			string = exporter.getExportSingleInteractionString(reaction, params).get(0);
			assertFalse(string.contains("trans-SNARE complex"));
			assertTrue(string.contains("RAB3A"));

		} catch (Exception e) {
			e.printStackTrace();
			fail("Exception occured");
		}

	}

	@Test
	public void testGetExportStringForOneReactionInGeneralWithOnlyOneElement() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export/reaction.xml", false);
			Reaction reaction = model.getReactionByReactionId("re11");
			model.getReactions().clear();
			model.addReaction(reaction);
			Set<Compartment> aliases = new HashSet<Compartment>();
			aliases.add((Compartment) model.getElementByElementId("ca1"));
			ExporterParameters params = new IExporterService.ExporterParameters()
					.model(model).fileType(ExportFileType.SIF).included("dopamine loaded synaptic vesicle");
			assertEquals(0, exporter.getExportSingleInteractionString(reaction, params).size());

			params = new IExporterService.ExporterParameters().model(model).fileType(ExportFileType.SIF);
			assertEquals(1, exporter.getExportSingleInteractionString(reaction, params).size());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetExportSingleInteractionString_text() {
		try {
			Model model = getModelForFile("testFiles/export_with_artifitial_comp.xml", false);
			for (Reaction reaction : model.getReactions()) {
				String str = exporter
						.getExportSingleInteractionString(
								reaction, new IExporterService.ExporterParameters().fileType(ExportFileType.TAB_SEPARATED).type(Species.class))
						.get(0);

				List<String> strings = new ArrayList<String>();
				strings.add(ReactionLineData.getByReactionType(reaction.getClass()).getCellDesignerString());
				strings.add(reaction.getIdReaction());
				for (ReactionNode node : reaction.getReactionNodes()) {
					strings.add(node.getElement().getName());

				}
				for (String string : strings) {
					assertTrue("Description \"" + str + "\" doesn't contain " + string + ". reactionId=" + reaction.getIdReaction(), str.contains(string));
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
			fail("Exception occured");
		}
	}

	@Test
	public void testGetExportComponentAndCompartments() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export/export_model.xml", false);
			new CreateHierarchyCommand(model, 7, 80).execute();
			ExporterParameters params = new IExporterService.ExporterParameters()
					.model(model).fileType(ExportFileType.TAB_SEPARATED).column(ExportColumn.NAME).column(ExportColumn.COMPONENT_NAME)
					.column(ExportColumn.COMPARTMENT_NAME);

			List<String> list = exporter.getExportStringForOneSpecies((Species) model.getElementByElementId("sa1"), params);
			String desc = list.get(0);
			String compartment = desc.split("\t")[2];
			assertEquals("c1", compartment);
			String component = desc.split("\t")[1];
			assertEquals("Test3", component);

			list = exporter.getExportStringForOneSpecies((Species) model.getElementByElementId("sa2"), params);
			desc = null;
			for (String string : list) {
				if (string.contains("Test 1")) {
					desc = string;
				}
			}
			compartment = desc.split("\t")[2];
			assertEquals("c2", compartment);
			component = desc.split("\t")[1];
			assertEquals("Test 1", component);

			for (String string : list) {
				if (string.contains("Test2")) {
					desc = string;
				}
			}
			compartment = desc.split("\t")[2];
			assertEquals("c2", compartment);
			component = desc.split("\t")[1];
			assertEquals("Test2", component);

			for (String string : list) {
				if (string.contains("Test3")) {
					desc = string;
				}
			}
			compartment = desc.split("\t")[2];
			assertEquals("c2", compartment);
			component = desc.split("\t")[1];
			assertEquals("Test3", component);

			list = exporter.getExportStringForOneSpecies((Species) model.getElementByElementId("sa3"), params);
			boolean ok = false;
			for (String string : list) {
				if (string.split("\t")[2].equals("null") && string.split("\t")[1].equals("Test2")) {
					ok = true;
				}
			}
			assertTrue(ok);

			list = exporter.getExportStringForOneSpecies((Species) model.getElementByElementId("sa4"), params);
			desc = list.get(0);
			compartment = desc.split("\t")[2];
			assertEquals("null", compartment);
			component = desc.split("\t")[1];
			assertEquals("Test3", component);

			list = exporter.getExportStringForOneSpecies((Species) model.getElementByElementId("sa5"), params);
			desc = list.get(0);
			compartment = desc.split("\t")[2];
			assertEquals("c3", compartment);
			component = desc.split("\t")[1];
			assertEquals("null", component);

			list = exporter.getExportStringForOneSpecies((Species) model.getElementByElementId("sa6"), params);
			desc = list.get(0);
			compartment = desc.split("\t")[2];
			assertEquals("null", compartment);
			component = desc.split("\t")[1];
			assertEquals("null", component);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetExportSpeciesStringForTabSeparatedFile2() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export_with_artifitial_comp.xml", false);
			ExportColumn columns[] = { ExportColumn.NAME, ExportColumn.ID, ExportColumn.COMPONENT_NAME, ExportColumn.COMPARTMENT_NAME, ExportColumn.TYPE };

			ExporterParameters params = new IExporterService.ExporterParameters()
					.model(model).column(columns).fileType(ExportFileType.TAB_SEPARATED).type(Object.class);

			// full export (all elements)
			String string = exporter.getExportSpeciesString(params);
			String lines[] = string.split("\n");
			assertTrue("Not enough elements in the result file", lines.length > 1);

			String response = lines[0];
			String textColumns[] = response.split("\t");
			assertEquals(columns.length, textColumns.length);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetExportCompartmentsString() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export/export_compartments.xml", false);

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).fileType(ExportFileType.TAB_SEPARATED).type(Object.class);

			// full export (all compartments)
			String string = exporter.getExportCompartmentsString(params);
			String lines[] = string.split("\n");
			assertTrue("Not enough elements in the result file", lines.length >= 2);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testAllExportColumnsSpecies() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export_with_artifitial_comp.xml", false);
			List<ExportColumn> columns = new ArrayList<ExportColumn>();
			for (ExportColumn column : ExportColumn.values()) {
				if (column.getClazz().isAssignableFrom(Species.class)) {
					columns.add(column);
				}
			}

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).column(columns).fileType(ExportFileType.TAB_SEPARATED);

			// full export (all elements)
			String string = exporter.getExportSpeciesString(params);
			assertNotNull(string);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testAllExportColumnsReaction() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export_with_artifitial_comp.xml", false);
			List<ExportColumn> columns = new ArrayList<ExportColumn>();
			for (ExportColumn column : ExportColumn.values()) {
				if (column.getClazz().isAssignableFrom(Reaction.class)) {
					columns.add(column);
				}
			}

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).column(columns).fileType(ExportFileType.TAB_SEPARATED);

			// full export (all elements)
			String string = exporter.getExportInteractionString(params);
			assertNotNull(string);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetTabSeparatedReaction() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export/reaction.xml", false);
			Set<Compartment> aliases = new HashSet<>();
			for (Compartment alias : model.getCompartments()) {
				if (alias.getName().equalsIgnoreCase("Dopamine loaded synaptic vesicle"))
					aliases.add(alias);
			}

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).//
					fileType(ExportFileType.TAB_SEPARATED).//
					column(ExportColumn.REACTION_ID).//
					column(ExportColumn.REACTION_TYPE).//
					type(Species.class);

			String string = exporter.getExportInteractionString(params);
			assertTrue(string.indexOf("STATE_TRANSITION	REACTANT") >= 0);
			assertTrue(string.indexOf("sa7") >= 0);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void test() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export/new_line_file.xml", false);
			List<ExportColumn> columns = new ArrayList<ExportColumn>();
			for (ExportColumn column : ExportColumn.values()) {
				if (column.getClazz().isAssignableFrom(Species.class)) {
					columns.add(column);
				}
			}

			ExporterParameters params = new IExporterService.ExporterParameters().model(model).column(columns).fileType(ExportFileType.TAB_SEPARATED);

			// full export (all elements)
			String string = exporter.getExportSpeciesString(params);
			String tmp[] = string.split("\n");
			for (String string2 : tmp) {
				assertTrue(string2, columns.size() <= string2.split("\t", -1).length);
			}
			assertNotNull(string);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testPubmedSummary() throws Exception {
		try {
			Model model = getModelForFile("testFiles/export/reaction.xml", false);
			List<PubmedAnnotatedElementsView> result = exporter.getPublicationModelSummary(model);
			assertNotNull(result);
			for (PubmedAnnotatedElementsView pubmedAnnotatedElementsView : result) {
				assertNotNull(pubmedAnnotatedElementsView.getArticle());
				assertNotNull(pubmedAnnotatedElementsView.getArticle().getTitle());
				assertTrue(pubmedAnnotatedElementsView.getElements().size() > 0);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}
}
