package lcsb.mapviewer.services.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamRelationType;
import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.layout.Layout;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelFullIndexed;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Complex;
import lcsb.mapviewer.model.map.species.Degraded;
import lcsb.mapviewer.model.map.species.Drug;
import lcsb.mapviewer.model.map.species.Gene;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.Ion;
import lcsb.mapviewer.model.map.species.Phenotype;
import lcsb.mapviewer.model.map.species.Protein;
import lcsb.mapviewer.model.map.species.Rna;
import lcsb.mapviewer.model.map.species.SimpleMolecule;
import lcsb.mapviewer.model.map.species.Species;
import lcsb.mapviewer.model.map.species.Unknown;
import lcsb.mapviewer.services.ServiceTestFunctions;
import lcsb.mapviewer.services.interfaces.ISearchService.CoordinatesSearchParams;
import lcsb.mapviewer.services.search.data.FullAliasView;
import lcsb.mapviewer.services.search.data.FullReactionView;
import lcsb.mapviewer.services.search.data.SearchElementResult;
import lcsb.mapviewer.services.utils.SearchIndexer;

public class SearchServiceTest extends ServiceTestFunctions {
	static Logger					logger										 = Logger.getLogger(SearchService.class);

	private Model					model											 = null;
	SearchIndexer					indexer										 = new SearchIndexer();

	Map<Class<?>, String>	speciesSearchPrefix				 = new HashMap<Class<?>, String>();
	Map<String, Class<?>>	speciesSearchReversePrefix = new HashMap<String, Class<?>>();

	@Before
	public void setUp() throws Exception {
		addSearchPrefix("complex", Complex.class);
		addSearchPrefix("degrded", Degraded.class);
		addSearchPrefix("drug", Drug.class);
		addSearchPrefix("gene", Gene.class);
		addSearchPrefix("ion", Ion.class);
		addSearchPrefix("phenotype", Phenotype.class);
		addSearchPrefix("protein", Protein.class);
		addSearchPrefix("rna", Rna.class);
		addSearchPrefix("molecule", SimpleMolecule.class);
		addSearchPrefix("unknown", Unknown.class);
	}

	private void addSearchPrefix(String prefix, Class<?> clazz) {
		speciesSearchPrefix.put(clazz, prefix);
		speciesSearchReversePrefix.put(prefix, clazz);
	}

	protected void createFullModel() throws Exception {
		model = getModelForFile("testFiles/searchModel.xml", false);
		model.setProject(new Project("unknown project"));
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testSearchByName() throws Exception {
		try {
			createFullModel();
			long count = searchHistoryDao.getCount();
			SearchElementResult result = searchService.searchByQuery(model, "reaction:re21", 50, null, "127.0.0.1");

			assertNotNull(result);
			assertEquals(4, result.getElements().size());
			FullReactionView reaction = (FullReactionView) result.getElements().get(0);
			assertTrue(reaction.getName().contains("re21"));

			long count2 = searchHistoryDao.getCount();

			assertEquals(count + 1, count2);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testSearchByName2() throws Exception {
		try {
			createFullModel();
			SearchElementResult result = searchService.searchByQuery(model, "BAD:BCL-2", 50, null, "127.0.0.1");

			assertNotNull(result);
			assertTrue(result.getElements().size() > 0);
			FullAliasView alias = (FullAliasView) result.getElements().get(0);
			assertEquals("BAD:BCL-2", alias.getName());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testSearchById() throws Exception {
		try {
			createFullModel();
			MiriamData md = new MiriamData(MiriamRelationType.BQ_BIOL_HAS_VERSION, MiriamType.CAS, "123");
			model.getElements().iterator().next().addMiriamData(md);
			SearchElementResult global = searchService.searchByQuery(model, "CAS:123", 50, null, "127.0.0.1");
			assertNotNull(global);
			assertEquals(1, global.size());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testSearchByName4() throws Exception {
		try {
			createFullModel();
			SearchElementResult result = searchService.searchByQuery(model, "BC", 50, true, "127.0.0.1");

			assertNotNull(result);
			assertEquals(0, result.size());

			result = searchService.searchByQuery(model, "BC", 50, false, "127.0.0.1");
			assertNotNull(result);
			assertTrue(result.size() > 0);

			result = searchService.searchByQuery(model, "BAD:BCL-2", 50, true, "127.0.0.1");
			assertNotNull(result);
			assertEquals(1, result.size());
			FullAliasView alias = (FullAliasView) result.get(0);
			assertEquals("BAD:BCL-2", alias.getName());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testQueryName() throws Exception {
		try {
			createFullModel();
			SearchElementResult result = searchService
					.searchByQuery(model, indexer.getQueryStringForIndex("reaction:re21", new HashSet<String>()), 50, null, "127.0.0.1");

			assertNotNull(result);
			assertEquals(4, result.size());
			FullReactionView reaction = (FullReactionView) result.get(0);
			assertTrue(reaction.getName().contains("re21"));
			assertTrue(result.get(1) instanceof FullAliasView);
			assertTrue(result.get(2) instanceof FullAliasView);
			assertTrue(result.get(3) instanceof FullAliasView);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testQueryName2() throws Exception {
		try {
			createFullModel();
			model.getModelData().setId(-13);
			SearchElementResult result = searchService
					.searchByQuery(model, indexer.getQueryStringForIndex("BCL-2", speciesSearchReversePrefix.keySet()), 50, null, "127.0.0.1");
			assertNotNull(result);
			assertTrue(result.size() > 0);
			FullAliasView alias = (FullAliasView) result.get(0);
			assertTrue("Invalid alias: " + alias.getName(), alias.getName().toUpperCase().contains("BCL"));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testQueryName3() throws Exception {
		try {
			createFullModel();
			SearchElementResult result = searchService.searchByQuery(model, "fdhgjkhdsfsdhfgfhsd", 50, null, "127.0.0.1");

			assertNotNull(result);
			assertEquals(0, result.size());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testQueryName4() throws Exception {
		try {
			createFullModel();
			SearchElementResult result = searchService
					.searchByQuery(model, indexer.getQueryStringForIndex("protein:BCL-2", speciesSearchReversePrefix.keySet()), 50, null, "127.0.0.1");
			assertNotNull(result);
			assertTrue(result.size() > 0);
			FullAliasView alias = (FullAliasView) result.get(0);
			assertTrue("Invalid alias: " + alias.getName(), alias.getName().toUpperCase().contains("BCL"));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testSearchClosest() throws Exception {
		try {
			model = getModelForFile("testFiles/graph_path_example3.xml", true);
			List<Object> elements = searchService.getClosestElements(model, new Point2D.Double(0, 0), 5);
			assertNotNull(elements);
			assertEquals(5, elements.size());
			assertTrue(elements.get(0) instanceof Species);
			Species sAlias = (Species) elements.get(0);
			assertEquals("s1", sAlias.getName());
			assertTrue(elements.get(1) instanceof Reaction);
			Reaction reaction = (Reaction) elements.get(1);
			assertEquals("re1", reaction.getIdReaction());
			assertTrue(elements.get(2) instanceof Species);
			sAlias = (Species) elements.get(2);
			assertEquals("s3", sAlias.getName());
			assertTrue(elements.get(3) instanceof Reaction);
			reaction = (Reaction) elements.get(3);
			assertEquals("re2", reaction.getIdReaction());
			assertTrue(elements.get(4) instanceof Species);
			sAlias = (Species) elements.get(4);
			assertEquals("s2", sAlias.getName());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testSearchClosestWithEmptyModel() throws Exception {
		try {
			model = new ModelFullIndexed(null);
			List<Object> elements = searchService.getClosestElements(model, new Point2D.Double(0, 0), 5);
			assertNotNull(elements);
			assertEquals(0, elements.size());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetAutocompleList() {
		try {
			Model model = new ModelFullIndexed(null);
			Species proteinAlias = new GenericProtein("a");
			proteinAlias.setName("PROT identifier");
			proteinAlias.getSynonyms().add("PROT synonym");
			proteinAlias.getSynonyms().add("another synonym");
			proteinAlias.setFullName("Protein common name");
			model.addElement(proteinAlias);

			SimpleMolecule alias = new SimpleMolecule("a2");
			alias.setName("Molecule2");
			model.addElement(alias);

			int oldVal = Configuration.getAutocompleteSize();
			Configuration.setAutocompleteSize(5);

			List<String> list = searchService.getAutocompleteList(model, "P");
			assertNotNull(list);
			assertEquals(4, list.size());
			assertTrue(list.contains("PROT identifier".toLowerCase()));
			assertTrue(list.contains("PROT synonym".toLowerCase()));
			assertTrue(list.contains("Protein common name".toLowerCase()));

			list = searchService.getAutocompleteList(model, "PROTEIN");
			assertNotNull(list);
			assertEquals(2, list.size());
			assertTrue(list.contains("Protein common name".toLowerCase()));

			list = searchService.getAutocompleteList(model, "m");
			assertNotNull(list);
			assertEquals(2, list.size());
			assertTrue(list.contains("Molecule2".toLowerCase()));

			list = searchService.getAutocompleteList(model, "blablabla");
			assertNotNull(list);
			assertEquals(0, list.size());

			Configuration.setAutocompleteSize(oldVal);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testGetAutocompleList2() {
		try {
			Model model = new ModelFullIndexed(null);
			GenericProtein proteinAlias = new GenericProtein("a1");
			proteinAlias.setName("PROT identifier");
			proteinAlias.setNotes("Synonyms: PROT synonym, another synonym\nName: Protein common name");
			model.addElement(proteinAlias);

			SimpleMolecule alias = new SimpleMolecule("a2");
			alias.setName("Molecule2");
			model.addElement(alias);

			List<String> list = searchService.getAutocompleteList(model, "PROT identifier");
			assertNotNull(list);
			assertEquals(1, list.size());
			assertTrue(list.get(0).equalsIgnoreCase("PROT identifier"));

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testSearchGenericProtein() throws Exception {
		try {
			Model model = getModelForFile("testFiles/generic.xml", true);

			List<String> list = searchService.getAutocompleteList(model, "generic");
			assertNotNull(list);
			assertEquals(2, list.size());
			assertTrue(list.contains("generic"));
			assertTrue(list.contains("generic protein"));

			SearchElementResult result = searchService.searchByQuery(model, "generic protein", 10, false, null);
			assertNotNull(result);

			assertTrue(result.size() > 0);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testSearchByCoordCompartment() throws Exception {
		try {
			Model model = getModelForFile("testFiles/graph_path_example3.xml", true);

			// we search in non-nested model
			SearchElementResult res = searchService.searchByCoordinates(new CoordinatesSearchParams().model(model).x(50).y(50));
			assertNotNull(res);
			assertEquals(0, res.size());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testSearchByCoordCompartment2() throws Exception {
		try {
			Model model = getModelForFile("testFiles/graph_path_example3.xml", true);

			Layout layout = new Layout();
			layout.setHierarchicalView(true);
			model.addLayout(layout);
			// we search in nested model
			SearchElementResult res = searchService.searchByCoordinates(new CoordinatesSearchParams().model(model).x(50).y(50).layoutIdentifier(0));
			assertNotNull(res);

			assertTrue(res.get(0) instanceof FullAliasView);
			FullAliasView view = (FullAliasView) res.get(0);
			assertEquals("Compartment", view.getType());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testSearchByCoordReaction() throws Exception {
		try {
			Model model = getModelForFile("testFiles/graph_path_example3.xml", true);

			// find compartment
			SearchElementResult res = searchService.searchByCoordinates(new CoordinatesSearchParams().model(model));
			// reaction is too far
			assertEquals(0, res.size());

			res = searchService.searchByCoordinates(new CoordinatesSearchParams().model(model).distance(1000));
			assertEquals(3, res.size());

			assertTrue(res.get(0) instanceof FullReactionView);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testSearchByCoordAlias() throws Exception {
		try {
			Model model = getModelForFile("testFiles/graph_path_example3.xml", true);

			SearchElementResult res = searchService.searchByCoordinates(new CoordinatesSearchParams().model(model).x(60).y(60));
			assertNotNull(res);
			assertTrue(res.size() > 0);

			assertTrue(res.get(0) instanceof FullAliasView);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testGetMiriamTypeForQuery() throws Exception {
		try {
			MiriamData md = new SearchService().getMiriamTypeForQuery("hgnc:SNCA");
			assertNotNull(md);
			assertEquals(MiriamType.HGNC, md.getDataType());
			assertEquals("SNCA", md.getResource());

			md = new SearchService().getMiriamTypeForQuery("reactome:REACT_15128");
			assertNotNull(md);
			assertEquals(MiriamType.REACTOME, md.getDataType());
			assertEquals("REACT_15128", md.getResource());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

	@Test
	public void testGetMiriamTypeForQuery2() throws Exception {
		try {
			MiriamData md = new SearchService().getMiriamTypeForQuery("UNKNOWN_HGNC:SNCA");
			assertNull(md);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

	}

}
