package lcsb.mapviewer.services.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.List;

import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.user.BasicPrivilege;
import lcsb.mapviewer.model.user.ObjectPrivilege;
import lcsb.mapviewer.model.user.PrivilegeType;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.services.ServiceTestFunctions;
import lcsb.mapviewer.services.view.PrivilegeView;
import lcsb.mapviewer.services.view.UserView;
import lcsb.mapviewer.services.view.UserView.UserProjectPrivilegeView;
import lcsb.mapviewer.services.view.UserViewFactory;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.annotation.Rollback;

@Rollback(true)
public class UserServiceTest extends ServiceTestFunctions {
	static Logger		logger = Logger.getLogger(UserServiceTest.class);

	@Autowired
	UserViewFactory	userViewFactory;

	@Before
	public void setUp() throws Exception {
		createUser();
		logService.setLoggedUser(user);
	}

	@After
	public void tearDown() throws Exception {
		userDao.delete(user);
	}

	@Test
	public void testLogin() {
		try {
			assertFalse(userService.login("john.doe", "incorrect password"));
			assertTrue(userService.login("john.doe", "passwd"));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testLoginWithNull() {
		try {
			assertFalse(userService.login("john.doe", null));
			assertFalse(userService.login(null, "passwd"));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testUserHasPrivilegeUserPrivilegeType1() {
		try {
			assertFalse(userService.userHasPrivilege(user, PrivilegeType.ADD_MAP));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testUserHasPrivilegeUserPrivilegeType3() {
		try {
			userService.userHasPrivilege(user, PrivilegeType.VIEW_PROJECT);
			fail("An exception should occure. VIEW_MAP must be connected with object");
		} catch (Exception e) {

		}
	}

	@Test
	public void testUserHasPrivilegeUserPrivilegeType2() {
		try {
			BasicPrivilege privilege = new BasicPrivilege();
			privilege.setLevel(1);
			privilege.setType(PrivilegeType.ADD_MAP);
			userService.setUserPrivilege(user, privilege);
			assertTrue(userService.userHasPrivilege(user, PrivilegeType.ADD_MAP));

		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknown exception");
		}
	}

	@Test
	public void testUserHasPrivilegeUserPrivilegeTypeObject() {
		try {
			Project project = new Project();
			projectDao.add(project);

			assertFalse(userService.userHasPrivilege(user, PrivilegeType.VIEW_PROJECT, project));
			try {
				userService.userHasPrivilege(user, PrivilegeType.ADD_MAP, project);
				fail("An exception should occure. ADD_MAP must not be connected with object");
			} catch (Exception e) {

			}

			try {
				userService.userHasPrivilege(user, PrivilegeType.VIEW_PROJECT, user);
				fail("An exception should occure. VIEW_MAP must be connected with Model type");
			} catch (Exception e) {

			}

			ObjectPrivilege privilege = new ObjectPrivilege();
			privilege.setLevel(1);
			privilege.setType(PrivilegeType.VIEW_PROJECT);
			privilege.setIdObject(project.getId());
			userService.setUserPrivilege(user, privilege);

			assertTrue(userService.userHasPrivilege(user, PrivilegeType.VIEW_PROJECT, project));

			privilege = new ObjectPrivilege(project, 0, PrivilegeType.VIEW_PROJECT, user);
			userService.setUserPrivilege(user, privilege);

			assertFalse(userService.userHasPrivilege(user, PrivilegeType.VIEW_PROJECT, project));

			projectDao.delete(project);
		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknown exception");
		}
	}

	@Test
	public void testUserHasPrivilegeUserPrivilegeTypeObject2() {
		try {
			userService.userHasPrivilege(user, PrivilegeType.VIEW_PROJECT, null);
			fail("Exception should occur");
		} catch (InvalidArgumentException e) {
		}
	}

	@Test
	public void testAddUser() throws Exception {
		try {
			long logCount = logDao.getCount();
			User user2 = new User();
			user2.setLogin("login");
			userService.addUser(user2);
			assertNotNull(user2.getId());
			long logCount2 = logDao.getCount();
			assertEquals("Log entry is missing for add user event", logCount + 1, logCount2);
			userDao.evict(user2);
			User user3 = userService.getUserById(user2.getId());
			assertNotNull(user3);
			userService.deleteUser(user3);
			user3 = userService.getUserById(user2.getId());
			assertNull(user3);

			long logCount3 = logDao.getCount();
			assertEquals("Log entry is missing for remove user event", logCount2 + 1, logCount3);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetAllUserList() throws Exception {
		try {
			User user2 = new User();
			user2.setLogin("login");
			userService.addUser(user2);

			Project project = new Project();
			projectDao.add(project);

			ObjectPrivilege privilege = new ObjectPrivilege();
			privilege.setLevel(1);
			privilege.setType(PrivilegeType.VIEW_PROJECT);
			privilege.setIdObject(project.getId());
			userService.setUserPrivilege(user2, privilege);

			List<UserView> list = userService.getAllUserRows();
			UserView currentUser = null;
			for (UserView userRow : list) {
				if (userRow.getIdObject() == user2.getId()) {
					currentUser = userRow;
				}
			}
			assertNotNull(currentUser);
			assertEquals(user2.getLogin(), currentUser.getLogin());

			PrivilegeType types[] = PrivilegeType.values();

			List<BasicPrivilege> basicTypes = new ArrayList<BasicPrivilege>();
			List<ObjectPrivilege> projectObjectTypes = new ArrayList<ObjectPrivilege>();
			for (PrivilegeType privilegeType : types) {
				if (privilegeType.getPrivilegeClassType().equals(BasicPrivilege.class)) {
					basicTypes.add(privilegeType.getPrivilegeClassType().newInstance());
				} else if (privilegeType.getPrivilegeClassType().equals(ObjectPrivilege.class)) {
					if (privilegeType.getPrivilegeObjectType().equals(Project.class)) {
						projectObjectTypes.add((ObjectPrivilege) privilegeType.getPrivilegeClassType().newInstance());
					}
				}
			}

			assertEquals(basicTypes.size(), currentUser.getBasicPrivileges().size());

			long projectCount = projectDao.getCount();

			assertEquals(projectCount, currentUser.getProjectPrivileges().size());

			UserProjectPrivilegeView currentProject = null;
			for (UserProjectPrivilegeView projectPrivilege : currentUser.getProjectPrivileges()) {
				assertEquals(projectObjectTypes.size(), projectPrivilege.getProjectPrivileges().size());
				if (projectPrivilege.getIdObject() == project.getId()) {
					currentProject = projectPrivilege;
				}
			}
			assertNotNull(currentProject);
			PrivilegeView currentPrivilege = null;
			for (PrivilegeView row : currentProject.getProjectPrivileges()) {
				if (row.getType() == PrivilegeType.VIEW_PROJECT) {
					currentPrivilege = row;
				}
			}
			assertNotNull(currentPrivilege);
			assertTrue(currentPrivilege.getSelected());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * Test update of a user that doesn't exist in db
	 */
	@Test
	public void testUpdateUserRow() {
		try {
			Project project = new Project();
			projectDao.add(project);

			UserView userRow = userService.createEmptyUserRow();

			for (PrivilegeView pRow : userRow.getBasicPrivileges()) {
				if (pRow.getType().equals(PrivilegeType.ADD_MAP)) {
					pRow.setSelected(true);
				}
			}

			for (PrivilegeView pRow : userRow.getProjectPrivilegeByProjectId(project.getId()).getProjectPrivileges()) {
				if (pRow.getType().equals(PrivilegeType.VIEW_PROJECT)) {
					pRow.setSelected(true);
				}
			}

			userService.updateUser(userRow);

			assertNotNull(userRow.getIdObject());
			assertTrue(userRow.getIdObject() != 0);

			User user2 = userDao.getById(userRow.getIdObject());

			assertTrue(userService.userHasPrivilege(user2, PrivilegeType.ADD_MAP));
			assertFalse(userService.userHasPrivilege(user2, PrivilegeType.PROJECT_MANAGEMENT));
			assertTrue(userService.userHasPrivilege(user2, PrivilegeType.VIEW_PROJECT, project));

			for (PrivilegeView pRow : userRow.getBasicPrivileges()) {
				if (pRow.getType().equals(PrivilegeType.ADD_MAP)) {
					pRow.setSelected(false);
				}
			}

			for (PrivilegeView pRow : userRow.getProjectPrivilegeByProjectId(project.getId()).getProjectPrivileges()) {
				if (pRow.getType().equals(PrivilegeType.VIEW_PROJECT)) {
					pRow.setSelected(false);
				}
			}

			userService.updateUser(userRow);

			assertFalse(userService.userHasPrivilege(user2, PrivilegeType.ADD_MAP));
			assertFalse(userService.userHasPrivilege(user2, PrivilegeType.VIEW_PROJECT, project));
		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknown exception");
		}
	}

	/**
	 * Test update of a user that exist in the db
	 * 
	 * @throws Exception
	 */
	@Test
	public void testUpdateUserRow2() throws Exception {
		String cryptedPassword = "passwd";

		try {
			Project project = new Project();
			projectDao.add(project);

			User user2 = new User();
			user2.setLogin("login");
			user2.setCryptedPassword(cryptedPassword);
			userService.addUser(user2);

			List<UserView> rows = userService.getAllUserRows();
			UserView userRow = null;
			for (UserView userRow2 : rows) {
				if (userRow2.getIdObject() == user2.getId())
					userRow = userRow2;
			}

			assertNotNull(userRow);

			assertFalse(userService.userHasPrivilege(user2, PrivilegeType.ADD_MAP));
			assertFalse(userService.userHasPrivilege(user2, PrivilegeType.VIEW_PROJECT, project));

			for (PrivilegeView pRow : userRow.getBasicPrivileges()) {
				if (pRow.getType().equals(PrivilegeType.ADD_MAP)) {
					pRow.setSelected(true);
				}
			}

			for (PrivilegeView pRow : userRow.getProjectPrivilegeByProjectId(project.getId()).getProjectPrivileges()) {
				if (pRow.getType().equals(PrivilegeType.VIEW_PROJECT)) {
					pRow.setSelected(true);
				}
			}

			userRow.setPassword("");

			userService.updateUser(userRow);

			User user3 = userDao.getById(user2.getId());

			// check if password didn't change
			assertEquals(cryptedPassword, user3.getCryptedPassword());

			assertTrue(userService.userHasPrivilege(user2, PrivilegeType.ADD_MAP));
			assertTrue(userService.userHasPrivilege(user2, PrivilegeType.VIEW_PROJECT, project));

			for (PrivilegeView pRow : userRow.getBasicPrivileges()) {
				if (pRow.getType().equals(PrivilegeType.ADD_MAP)) {
					pRow.setSelected(false);
				}
			}

			for (PrivilegeView pRow : userRow.getProjectPrivilegeByProjectId(project.getId()).getProjectPrivileges()) {
				if (pRow.getType().equals(PrivilegeType.VIEW_PROJECT)) {
					pRow.setSelected(false);
				}
			}

			userRow.setPassword("new passwd");
			userService.updateUser(userRow);

			user3 = userDao.getById(user2.getId());

			// check if password changed
			assertFalse(cryptedPassword.equals(user3.getCryptedPassword()));

			assertFalse(userService.userHasPrivilege(user2, PrivilegeType.ADD_MAP));
			assertFalse(userService.userHasPrivilege(user2, PrivilegeType.VIEW_PROJECT, project));
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testRemoveUserRow() throws Exception {
		try {
			UserView userRow = userService.createEmptyUserRow();
			userService.deleteUser(userRow);

			User user2 = new User();
			user2.setLogin("login");
			userService.addUser(user2);

			List<UserView> rows = userService.getAllUserRows();
			userRow = null;
			for (UserView userRow2 : rows) {
				if (userRow2.getIdObject() == user2.getId())
					userRow = userRow2;
			}

			assertNotNull(userRow);

			userService.deleteUser(userRow);

			rows = userService.getAllUserRows();

			userRow = null;
			for (UserView userRow2 : rows) {
				if (userRow2.getIdObject() == user2.getId())
					userRow = userRow2;
			}

			assertNull(userRow);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCreateEmptyUserRow() {
		try {
			Project project = new Project();
			projectDao.add(project);

			UserView row = userService.createEmptyUserRow();

			assertNotNull(row);

			long projectCount = projectDao.getCount();

			assertEquals(projectCount, row.getProjectPrivileges().size());
			assertNotNull(row.getProjectPrivilegeByProjectId(project.getId()));

			for (UserProjectPrivilegeView upRow : row.getProjectPrivileges()) {
				assertTrue(userViewFactory.getObjectTypes().size() >= upRow.getProjectPrivileges().size());
			}
			assertEquals(userViewFactory.getBasicTypes().size(), row.getBasicPrivileges().size());

		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknown exception");
		}
	}

}
