package lcsb.mapviewer.services.search.db.mirna;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.google.gson.Gson;

import lcsb.mapviewer.annotation.data.MiRNA;
import lcsb.mapviewer.annotation.data.Target;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.services.overlay.IconManager;
import lcsb.mapviewer.services.overlay.IconType;
import lcsb.mapviewer.services.search.SearchResultFactory;
import lcsb.mapviewer.services.search.data.ElementIdentifier;
import lcsb.mapviewer.services.search.db.DrugTargetViewVisibilityComparator;
import lcsb.mapviewer.services.search.db.TargetView;
import lcsb.mapviewer.services.search.db.TargetViewFactory;

/**
 * Factory class for {@link MiRNAView} class.
 * 
 * @author Ayan Rota
 * 
 */
public class MiRNAViewFactory extends SearchResultFactory<MiRNA, MiRNAView> {

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger			logger = Logger.getLogger(MiRNAViewFactory.class);

	/**
	 * Factory object for {@link TargetView} elements.
	 */
	@Autowired
	private TargetViewFactory	drugTargetViewFactory;

	@Override
	public MiRNAView create(MiRNA miRNA) {
		return create(miRNA, null, 0);
	}

	/**
	 * Creates {@link MiRNAView}.
	 * 
	 * @param miRNA
	 *          original MiRNA from which the data will be initialized
	 * @param model
	 *          model where the view will be presented
	 * @param set
	 *          which set of icons should be used
	 * 
	 * @return {@link MiRNAView} object for given miRNA. links are placed
	 *         according to model given in the parameter.
	 */
	public MiRNAView create(MiRNA miRNA, Model model, int set) {
		MiRNAView miRNAView = new MiRNAView(miRNA);
		if (miRNA == null) {
			return miRNAView;
		}

		miRNAView.setName(miRNA.getName());

		List<TargetView> targetsRows = new ArrayList<>();
		int differentNames = 0;
		for (Target geneEntry : miRNA.getTargets()) {
			targetsRows.add(drugTargetViewFactory.create(geneEntry, model));
		}

		Collections.sort(targetsRows, new DrugTargetViewVisibilityComparator());

		for (TargetView geneTargetView : targetsRows) {
			if (geneTargetView.getSelectable()) {
				String icon = IconManager.getInstance().getIconForIndex(differentNames++, IconType.MI_RNA, set);
				geneTargetView.setIcon(icon);
				geneTargetView.setSelected(true);
			}
		}
		miRNAView.setTargetRows(targetsRows);
		return miRNAView;

	}

	@Override
	public String createGson(MiRNAView object) {
		return new Gson().toJson(object);
	}

	@Override
	public List<ElementIdentifier> searchResultToElementIdentifier(MiRNAView searchResult, Model inputModel) {
		List<ElementIdentifier> result = new ArrayList<>();

		List<Model> models = new ArrayList<>();
		models.addAll(inputModel.getSubmodels());
		models.add(inputModel);
		for (Model model : models) {
			for (TargetView target : searchResult.getTargetRows()) {
				if (target.getSelected() && target.getIcon() != null) {
					for (Element element : model.getElements()) {
						if (elementMatch(target, element)) {
							result.add(new ElementIdentifier(element, target.getIcon()));
						}
					}
				}
			}
		}
		return result;
	}

}
